@@ -1,5 +1,5 @@
-#!/bin/bash
-# $Id: m4-1.4.4.sh 2837 2007-02-19 05:58:22Z dclark $
+#!/bin/sh
+# $Id: m4-1.4.4.sh 3451 2007-07-13 21:28:25Z dclark $
 
 ENCAP_PKGNAME="m4-1.4.4"
 PATH="$PATH:/usr/local/bin"
@@ -18,7 +18,7 @@ cat > ${ENCAP_PKGNAME}.ep << EOF
 
 <environment
         variable="CC"
-        value="gcc"
+        value="gcc -static-libgcc"
         type="set"
 />
 
@@ -35,7 +35,8 @@ cat > ${ENCAP_PKGNAME}.ep << EOF
 />
 
 <source
-url="http://www.opensysadmin.com/mirror/m4/m4-1.4.4.tar.gz
+url="http://encapsrcdist/mirror/m4/m4-1.4.4.tar.gz
+     http://mirror.opensysadmin.com/m4/m4-1.4.4.tar.gz
      http://ftp.gnu.org/gnu/m4/m4-1.4.4.tar.gz"
 >
 
