@@ -1,8 +1,8 @@
 #!/bin/sh -e
 
-# $Id: makedist.sh 3064 2007-04-26 20:47:05Z dclark $
+# $Id: makedist.sh 3473 2007-07-18 17:04:16Z dclark $
 
-# Script to create the 
+# Script to create the
 # bcfg2-<sitename>-<ver>-<arch>-<os>.run
 # (example: bcfg2-nasa-1-rs6000-aix5.3.0.run)
 # one-step install bcfg2 client distribution file
@@ -25,7 +25,7 @@ SITEVER="`echo $BSEP | awk -F\- '{print $4}'`"
 
 EPEP="`basename $EPDIR/m4-*-encap-*.tar.gz`"
 ARCH="`echo $EPEP | awk -F\- '{print $4}'`"
-OS="`echo $EPEP | awk -F\- '{print $5}' | awk -F. '{print $1}'`"
+OS="`echo $EPEP | awk -F\- '{print $5}' | sed s:\.tar\.gz$::g`"
 
 # Make temporary directory $DISTDIR from which distribution will be created
 cd $MDDIR
@@ -50,6 +50,7 @@ tar -cf $DISTDIR/epkg.tar $EPKGDIR/*
 BCFG2="`basename $EPDIR/bcfg2-[0-9].[0-9]*-encap-*.tar.gz`"
 BCFG2_ELEMENTTREE="`basename $EPDIR/bcfg2-elementtree-*-encap-*.tar.gz`"
 BCFG2_M2CRYPTO="`basename $EPDIR/bcfg2-m2crypto-*-encap-*.tar.gz`"
+BCFG2_OPENSSL="`basename $EPDIR/bcfg2-openssl-*-encap-*.tar.gz`"
 BCFG2_PYTHON="`basename $EPDIR/bcfg2-python-[0-9].[0-9]*-encap-*.tar.gz`"
 OSTIARTY="`basename $EPDIR/ostiary-[0-9].[0-9]*-encap-*.tar.gz`"
 RUNIT="`basename $EPDIR/runit-[0-9].[0-9]*-encap-*.tar.gz`"
@@ -59,7 +60,7 @@ if [ "${BCFG2_PYTHON_APT_TMP}x" != 'bcfg2-python-apt-*-encap-*.tar.gzx' ]; then
     BCFG2_PYTHON_APT="$BCFG2_PYTHON_APT_TMP"
 fi
 
-BCFG2DEPS="$BCFG2_ELEMENTTREE $BCFG2_M2CRYPTO $BCFG2_PYTHON $BCFG2_PYTHON_APT"
+BCFG2DEPS="$BCFG2_ELEMENTTREE $BCFG2_M2CRYPTO $BCFG2_OPENSSL $BCFG2_PYTHON $BCFG2_PYTHON_APT"
 
 FILES="$BCFG2DEPS $BCFG2 $RUNIT $OSTIARTY"
 
@@ -82,7 +83,7 @@ fi
 ######################################################################
 cat >> $DISTDIR/setup.sh << EOF
 
-# \$Id: makedist.sh 3064 2007-04-26 20:47:05Z dclark $
+# \$Id: makedist.sh 3473 2007-07-18 17:04:16Z dclark $
 
 # This is the script that is run by makeself after it extracts all the files
 # from the .run distribution. It installs epkg, and then all the encaps in the
@@ -131,18 +132,18 @@ LOC_OST_ACTION_RE='^ACTION=\"-bcfg2-'
 # Check to see if passwords are set
 printf "Checking to see if password is set in \'\${LOC_BCFG2_CONF}\'... "
 grep "\${LOC_BCFG2_RE}" \$LOC_BCFG2_CONF >/dev/null && LOC_BCFG2_SET="no"
-if [ "\${LOC_BCFG2_SET}x" = "nox" ]; then 
+if [ "\${LOC_BCFG2_SET}x" = "nox" ]; then
     printf "no\n"
-else 
+else
     printf "yes\n"
 fi
 
 printf "Checking to see if passwords are set in \'\${LOC_OST_CFG}\'... "
 grep "\${LOC_OST_KILL_RE}" \$LOC_OST_CFG >/dev/null && LOC_OST_SET="no"
 grep "\${LOC_OST_ACTION_RE}" \$LOC_OST_CFG >/dev/null && LOC_OST_SET="no"
-if [ "\${LOC_OST_SET}x" = "nox" ]; then 
+if [ "\${LOC_OST_SET}x" = "nox" ]; then
     printf "no\n"
-else 
+else
     printf "yes\n"
 fi
 
@@ -190,7 +191,7 @@ fi
 
 # Sed quoting function - quote the &, :, ' and \ characters
 sedquote() {
-    i=1 
+    i=1
     while [ \$i -le \`${EXPR} length \$1\` ]; do
         c=\`${EXPR} substr \$1 \$i 1\`
         if [ "\${c}x" = "&x" -o "\${c}x" = ":x" -o "\${c}x" = "'x" -o "\${c}x" = "\\\\x" ]; then
@@ -241,14 +242,14 @@ chmod 600 \${LOC_OST_CFG}*
 if [ -x /usr/local/bin/sv ]; then
     for LOC_SERVICE in bcfg2-client bcfg2-server ostiary; do
         if [ -h /usr/local/var/service/\${LOC_SERVICE} ]; then
-            printf "Restarting runit service \${LOC_SERVICE}...\n" 
+            printf "Restarting runit service \${LOC_SERVICE}...\n"
             /usr/local/bin/sv restart /usr/local/var/service/\${LOC_SERVICE}
             sleep 2
             /usr/local/bin/sv status /usr/local/var/service/\${LOC_SERVICE}
         fi
     done
 fi
-    
+
 exit 0
 
 EOF
