@@ -1,7 +1,11 @@
-'''Frame is the Client Framework that verifies and installs entries, and generates statistics'''
-__revision__ = '$Revision: 4933 $'
+"""
+Frame is the Client Framework that verifies and
+installs entries, and generates statistics
+"""
+__revision__ = '$Revision: 5499 $'
 
-import logging, time, types
+import logging
+import time
 import Bcfg2.Client.Tools
 
 def cmpent(ent1, ent2):
@@ -24,6 +28,7 @@ def promptFilter(prompt, entries):
             if raw_input(iprompt) in ['y', 'Y']:
                 ret.append(entry)
         except:
+            print("Error while reading input")
             continue
     return ret
 
@@ -53,13 +58,13 @@ class Frame:
         self.logger = logging.getLogger("Bcfg2.Client.Frame")
         for driver in drivers[:]:
             if driver not in Bcfg2.Client.Tools.drivers and \
-                   isinstance(driver, types.StringType):
+                   isinstance(driver, str):
                 self.logger.error("Tool driver %s is not available" % driver)
                 drivers.remove(driver)
-                
+
         tclass = {}
         for tool in drivers:
-            if not isinstance(tool, types.StringType):
+            if not isinstance(tool, str):
                 tclass[time.time()] = tool
             tool_class = "Bcfg2.Client.Tools.%s" % tool
             try:
@@ -72,38 +77,40 @@ class Frame:
                 self.logger.error("Tool %s unexpectedly failed to load" % tool,
                                   exc_info=1)
 
-        for tool in tclass.values():
+        for tool in list(tclass.values()):
             try:
                 self.tools.append(tool(self.logger, setup, config))
             except Bcfg2.Client.Tools.toolInstantiationError:
                 continue
             except:
-                self.logger.error("Failed to instantiate tool %s" % (tool), exc_info=1)
+                self.logger.error("Failed to instantiate tool %s" % \
+                                  (tool), exc_info=1)
 
         for tool in self.tools[:]:
             for conflict in getattr(tool, 'conflicts', []):
                 [self.tools.remove(item) for item in self.tools \
-                 if item.__name__ == conflict]
+                 if item.name == conflict]
 
         self.logger.info("Loaded tool drivers:")
-        self.logger.info([tool.__name__ for tool in self.tools])
+        self.logger.info([tool.name for tool in self.tools])
         if not self.dryrun and not self.setup['bundle']:
             for cfile in [cfl for cfl in config.findall(".//ConfigFile") \
                           if cfl.get('name') in self.__important__]:
-                tl= [t for t in self.tools if t.handlesEntry(cfile) \
+                tl = [t for t in self.tools if t.handlesEntry(cfile) \
                      and t.canVerify(cfile)]
                 if tl:
                     if not tl[0].VerifyConfigFile(cfile, []):
                         if self.setup['interactive'] and not \
-                               promptFilter("Install %s: %s? (y/N):", [cfile]): 
-                            continue 
+                               promptFilter("Install %s: %s? (y/N):", [cfile]):
+                            continue
                         try:
                             self.states[cfile] = tl[0].InstallConfigFile(cfile)
                         except:
                             self.logger.error("Unexpected tool failure",
                                               exc_info=1)
         # find entries not handled by any tools
-        problems = [entry for struct in config for entry in struct if entry not in self.handled]
+        problems = [entry for struct in config for \
+                    entry in struct if entry not in self.handled]
 
         if problems:
             self.logger.error("The following entries are not handled by any tool:")
@@ -111,6 +118,7 @@ class Frame:
                                              entry.get('name')) for entry in problems])
             self.logger.error("")
         entries = [(entry.tag, entry.get('name')) for struct in config for entry in struct]
+        pkgs = [(entry.get('name'), entry.get('origin')) for struct in config for entry in struct if entry.tag == 'Package']
         multi = []
         for entry in entries[:]:
             if entries.count(entry) > 1:
@@ -120,20 +128,29 @@ class Frame:
             self.logger.debug("The following entries are included multiple times:")
             self.logger.debug(["%s:%s" % entry for entry in multi])
             self.logger.debug("")
-            
-                    
+        if pkgs:
+            self.logger.debug("The following packages are specified in bcfg2:")
+            self.logger.debug([pkg[0] for pkg in pkgs if pkg[1] == None])
+            self.logger.debug("The following packages are prereqs added by Packages:")
+            self.logger.debug([pkg[0] for pkg in pkgs if pkg[1] == 'Packages'])
+
+
     def __getattr__(self, name):
         if name in ['extra', 'handled', 'modified', '__important__']:
             ret = []
             for tool in self.tools:
                 ret += getattr(tool, name)
             return ret
-        elif self.__dict__.has_key(name):
+        elif name in self.__dict__:
             return self.__dict__[name]
-        raise AttributeError, name
+        raise AttributeError(name)
 
     def Inventory(self):
-        '''Verify all entries, find extra entries, and build up workqueues'''
+        '''
+           Verify all entries,
+           find extra entries,
+           and build up workqueues
+        '''
         # initialize all states
         for struct in self.config.getchildren():
             for entry in struct.getchildren():
@@ -142,7 +159,7 @@ class Frame:
             try:
                 tool.Inventory(self.states)
             except:
-                self.logger.error("%s.Inventory() call failed:" % tool.__name__, exc_info=1)
+                self.logger.error("%s.Inventory() call failed:" % tool.name, exc_info=1)
 
     def Decide(self):
         '''Set self.whitelist based on user interaction'''
@@ -152,13 +169,17 @@ class Frame:
             if self.setup['remove'] == 'all':
                 self.removal = self.extra
             elif self.setup['remove'] == 'services':
-                self.removal = [entry for entry in self.extra if entry.tag == 'Service']
+                self.removal = [entry for entry in self.extra \
+                                if entry.tag == 'Service']
             elif self.setup['remove'] == 'packages':
-                self.removal = [entry for entry in self.extra if entry.tag == 'Package']
+                self.removal = [entry for entry in self.extra \
+                                if entry.tag == 'Package']
 
-        candidates = [entry for entry in self.states if not self.states[entry]]
-        self.whitelist = [entry for entry in self.states if not self.states[entry]]
-        # Need to process decision stuff early, so that dryrun mode works with it
+        candidates = [entry for entry in self.states \
+                      if not self.states[entry]]
+        self.whitelist = [entry for entry in self.states \
+                          if not self.states[entry]]
+        # Need to process decision stuff early so that dryrun mode works with it
         if self.setup['decision'] == 'whitelist':
             dwl = self.setup['decision_list']
             w_to_rem = [e for e in self.whitelist \
@@ -166,7 +187,8 @@ class Frame:
             if w_to_rem:
                 self.logger.info("In whitelist mode: suppressing installation of:")
                 self.logger.info(["%s:%s" % (e.tag, e.get('name')) for e in w_to_rem])
-                self.whitelist = [x for x in self.whitelist if x not in w_to_rem]
+                self.whitelist = [x for x in self.whitelist \
+                                  if x not in w_to_rem]
 
         elif self.setup['decision'] == 'blacklist':
             b_to_rem = [e for e in self.whitelist \
@@ -175,7 +197,7 @@ class Frame:
                 self.logger.info("In blacklist mode: suppressing installation of:")
                 self.logger.info(["%s:%s" % (e.tag, e.get('name')) for e in b_to_rem])
                 self.whitelist = [x for x in self.whitelist if x not in b_to_rem]
-            
+
         if self.dryrun:
             if self.whitelist:
                 self.logger.info("In dryrun mode: suppressing entry installation for:")
@@ -191,10 +213,19 @@ class Frame:
         # Here is where most of the work goes
         # first perform bundle filtering
         if self.setup['bundle']:
+            all_bundle_names = [b.get('name') for b in
+                                self.config.findall('./Bundle')]
+            # warn if non-existent bundle given
+            for bundle in self.setup['bundle']:
+                if bundle not in all_bundle_names:
+                    self.logger.info("Warning: Bundle %s not found" % bundle)
             bundles = [b for b in self.config.findall('./Bundle') \
                        if b.get('name') in self.setup['bundle']]
             self.whitelist = [e for e in self.whitelist if \
                               True in [e in b for b in bundles]]
+        elif self.setup['indep']:
+            bundles = [nb for nb in self.config.getchildren() if nb.tag != \
+                       'Bundle']
         else:
             bundles = self.config.getchildren()
 
@@ -223,8 +254,8 @@ class Frame:
                     [self.whitelist.remove(ent) for ent in b_to_remv]
 
         if self.setup['interactive']:
-            self.whitelist = promptFilter(prompt, self.whitelist) 
-            self.removal = promptFilter(rprompt, self.removal) 
+            self.whitelist = promptFilter(prompt, self.whitelist)
+            self.removal = promptFilter(rprompt, self.removal)
 
         for entry in candidates:
             if entry not in self.whitelist:
@@ -239,7 +270,7 @@ class Frame:
             try:
                 tool.Install(handled, self.states)
             except:
-                self.logger.error("%s.Install() call failed:" % tool.__name__, exc_info=1)
+                self.logger.error("%s.Install() call failed:" % tool.name, exc_info=1)
 
     def Install(self):
         '''Install all entries'''
@@ -259,7 +290,7 @@ class Frame:
                 try:
                     tool.Inventory(self.states, [bundle])
                 except:
-                    self.logger.error("%s.Inventory() call failed:" % tool.__name__, exc_info=1)
+                    self.logger.error("%s.Inventory() call failed:" % tool.name, exc_info=1)
             clobbered = [entry for bundle in mbundles for entry in bundle \
                          if not self.states[entry] and entry not in self.blacklist]
             if clobbered:
@@ -270,6 +301,10 @@ class Frame:
                     self.DispatchInstallCalls(clobbered)
 
         for bundle in self.config.findall('.//Bundle'):
+            if self.setup['bundle'] and \
+                   bundle.get('name') not in self.setup['bundle']:
+                # prune out unspecified bundles when running with -b
+                continue
             for tool in self.tools:
                 try:
                     if bundle in mbundles:
@@ -278,8 +313,8 @@ class Frame:
                         tool.BundleNotUpdated(bundle, self.states)
                 except:
                     self.logger.error("%s.BundleNotUpdated() call failed:" % \
-                                      (tool.__name__), exc_info=1)
-                
+                                      (tool.name), exc_info=1)
+
     def Remove(self):
         '''Remove extra entries'''
         for tool in self.tools:
@@ -288,25 +323,25 @@ class Frame:
                 try:
                     tool.Remove(extras)
                 except:
-                    self.logger.error("%s.Remove() failed" % tool.__name__, exc_info=1)
+                    self.logger.error("%s.Remove() failed" % tool.name, exc_info=1)
 
     def CondDisplayState(self, phase):
         '''Conditionally print tracing information'''
         self.logger.info('\nPhase: %s' % phase)
-        self.logger.info('Correct entries:\t%d' % self.states.values().count(True))
-        self.logger.info('Incorrect entries:\t%d' % self.states.values().count(False))
-        if phase == 'final' and self.states.values().count(False):
+        self.logger.info('Correct entries:\t%d' % list(self.states.values()).count(True))
+        self.logger.info('Incorrect entries:\t%d' % list(self.states.values()).count(False))
+        if phase == 'final' and list(self.states.values()).count(False):
             self.logger.info(["%s:%s" % (entry.tag, entry.get('name')) for \
                               entry in self.states if not self.states[entry]])
-        self.logger.info('Total managed entries:\t%d' % len(self.states.values()))
+        self.logger.info('Total managed entries:\t%d' % len(list(self.states.values())))
         self.logger.info('Unmanaged entries:\t%d' % len(self.extra))
         if phase == 'final' and self.setup['extra']:
             self.logger.info(["%s:%s" % (entry.tag, entry.get('name')) \
                               for entry in self.extra])
-                
+
         self.logger.info("")
 
-        if ((self.states.values().count(False) == 0) and not self.extra):
+        if ((list(self.states.values()).count(False) == 0) and not self.extra):
             self.logger.info('All entries correct.')
 
     def ReInventory(self):
@@ -334,13 +369,15 @@ class Frame:
     def GenerateStats(self):
         '''Generate XML summary of execution statistics'''
         feedback = Bcfg2.Client.XML.Element("upload-statistics")
-        stats = Bcfg2.Client.XML.SubElement(feedback, \
-                                            'Statistics', total=str(len(self.states)),
-                                            client_version=__revision__, version='2.0',
+        stats = Bcfg2.Client.XML.SubElement(feedback,
+                                            'Statistics',
+                                            total=str(len(self.states)),
+                                            client_version=__revision__,
+                                            version='2.0',
                                             revision=self.config.get('revision', '-1'))
-        good = len([key for key, val in self.states.iteritems() if val])
+        good = len([key for key, val in list(self.states.items()) if val])
         stats.set('good', str(good))
-        if len([key for key, val in self.states.iteritems() if not val]) == 0:
+        if len([key for key, val in list(self.states.items()) if not val]) == 0:
             stats.set('state', 'clean')
         else:
             stats.set('state', 'dirty')
@@ -357,7 +394,7 @@ class Frame:
 
         timeinfo = Bcfg2.Client.XML.Element("OpStamps")
         feedback.append(stats)
-        for (event, timestamp) in self.times.iteritems():
+        for (event, timestamp) in list(self.times.items()):
             timeinfo.set(event, str(timestamp))
         stats.append(timeinfo)
         return feedback
