@@ -1,18 +1,21 @@
 # This is the bcfg2 support for chkconfig
-# $Id: Chkconfig.py 4940 2008-10-10 02:13:25Z desai $
+# $Id: Chkconfig.py 5408 2009-08-16 19:57:11Z solj $
 
 '''This is chkconfig support'''
-__revision__ = '$Revision: 4940 $'
+__revision__ = '$Revision: 5408 $'
 
 import Bcfg2.Client.Tools, Bcfg2.Client.XML
 
 class Chkconfig(Bcfg2.Client.Tools.SvcTool):
     '''Chkconfig support for Bcfg2'''
-    __name__ = 'Chkconfig'
+    name = 'Chkconfig'
     __execs__ = ['/sbin/chkconfig']
     __handles__ = [('Service', 'chkconfig')]
     __req__ = {'Service': ['name', 'status']}
 
+    def get_svc_command(self, service, action):
+        return "/sbin/service %s %s" % (service.get('name'), action)
+
     def VerifyService(self, entry, _):
         '''Verify Service status for entry'''
         try:
@@ -44,7 +47,7 @@ class Chkconfig(Bcfg2.Client.Tools.SvcTool):
         else:
             status = (len(onlevels) == 0)
 
-        if entry.get('supervised', 'false') == 'true':
+        if entry.get('mode', 'default') == 'supervised':
             pstatus, pout = self.cmd.run('/sbin/service %s status' % \
                                          entry.get('name'))
             if pstatus:
@@ -64,12 +67,12 @@ class Chkconfig(Bcfg2.Client.Tools.SvcTool):
 
     def InstallService(self, entry):
         '''Install Service entry'''
-        rcmd = "/sbin/chkconfig %s %s" 
+        rcmd = "/sbin/chkconfig %s %s"
         self.cmd.run("/sbin/chkconfig --add %s"%(entry.attrib['name']))
         self.logger.info("Installing Service %s" % (entry.get('name')))
         pass1 = True
         if entry.get('status') == 'off':
-            rc = self.cmd.run(rcmd % (entry.get('name'), entry.get('status')) + " --level 1")[0]
+            rc = self.cmd.run(rcmd % (entry.get('name'), entry.get('status')) + " --level 0123456")[0]
             pass1 = rc == 0
         rc = self.cmd.run(rcmd % (entry.get('name'), entry.get('status')))[0]
         return pass1 and rc == 0
