@@ -1,16 +1,15 @@
 '''Debian Init Support for Bcfg2'''
-__revision__ = '$Revision: 4423 $'
+__revision__ = '$Revision: 5408 $'
 
 import glob, os, re
 import Bcfg2.Client.Tools
 
 class DebInit(Bcfg2.Client.Tools.SvcTool):
     '''Debian Service Support for Bcfg2'''
-    __name__ = 'DebInit'
-    __execs__ = ['/usr/sbin/update-rc.d']
+    name = 'DebInit'
+    __execs__ = ['/usr/sbin/update-rc.d', '/usr/sbin/invoke-rc.d']
     __handles__ = [('Service', 'deb')]
     __req__ = {'Service': ['name', 'status']}
-    __svcrestart__ = 'restart'
     svcre = re.compile("/etc/.*/(?P<action>[SK])(?P<sequence>\d+)(?P<name>\S+)")
 
     # implement entry (Verify|Install) ops
@@ -23,7 +22,7 @@ class DebInit(Bcfg2.Client.Tools.SvcTool):
             kill_sequence = 100 - start_sequence
         else:
             start_sequence = None
-        
+
         for filename in rawfiles:
             match = self.svcre.match(filename)
             if not match:
@@ -60,7 +59,7 @@ class DebInit(Bcfg2.Client.Tools.SvcTool):
         except OSError:
             self.logger.debug("Init script for service %s does not exist" % entry.get('name'))
             return False
-        
+
         if entry.get('status') == 'off':
             self.cmd.run("/usr/sbin/invoke-rc.d %s stop" % (entry.get('name')))
             cmdrc = self.cmd.run("/usr/sbin/update-rc.d -f %s remove" % entry.get('name'))[0]
@@ -94,18 +93,5 @@ class DebInit(Bcfg2.Client.Tools.SvcTool):
         # Extra services need to be reflected in the config
         return
 
-    def BundleUpdated(self, bundle, states):
-        '''The Bundle has been updated'''
-        for entry in bundle:
-            if self.handlesEntry(entry):
-                command = "/usr/sbin/invoke-rc.d %s" % entry.get('name')
-                if entry.get('status') == 'on' and not self.setup['build']:
-                    self.logger.debug('Restarting service %s' % entry.get('name'))
-                    rc = self.cmd.run('%s %s' % (command, \
-                                                 entry.get('reload', self.__svcrestart__)))[0]
-                else:
-                    self.logger.debug('Stopping service %s' % entry.get('name'))
-                    rc = self.cmd.run('%s stop' %  command)[0]
-                if rc:
-                    self.logger.error("Failed to restart service %s" % (entry.get('name')))
-
+    def get_svc_command(self, service, action):
+        return '/usr/sbin/invoke-rc.d %s %s' % (service.get('name'), action)
