@@ -1,5 +1,5 @@
 '''This is the bcfg2 tool for the FreeBSD package system.'''
-__revision__ = '$Rev: 4602 $'
+__revision__ = '$Rev: 4999 $'
 
 # TODO
 # - actual package installation
@@ -11,7 +11,7 @@ import Bcfg2.Client.Tools
 class FreeBSDPackage(Bcfg2.Client.Tools.PkgTool):
     '''The FreeBSD toolset implements package operations and inherits
     the rest from Toolset.Toolset'''
-    __name__ = 'FreeBSDPackage'
+    name = 'FreeBSDPackage'
     __execs__ = ['/usr/sbin/pkg_add', '/usr/sbin/pkg_info']
     __handles__ = [('Package', 'freebsdpkg')]
     __req__ = {'Package': ['name', 'version']}
@@ -29,11 +29,11 @@ class FreeBSDPackage(Bcfg2.Client.Tools.PkgTool):
                 self.installed[name] = version
 
     def VerifyPackage(self, entry, modlist):
-        if not entry.attrib.has_key('version'):
+        if not 'version' in entry.attrib:
             self.logger.info("Cannot verify unversioned package %s" %
                (entry.attrib['name']))
             return False
-        if self.installed.has_key(entry.attrib['name']):
+        if entry.attrib['name'] in self.installed:
             if self.installed[entry.attrib['name']] == entry.attrib['version']:
                 # TODO: verfification
                 return True
