@@ -1,31 +1,52 @@
 '''All POSIX Type client support for Bcfg2'''
-__revision__ = '$Revision: 4923 $'
+__revision__ = '$Revision: 5514 $'
 
+from datetime import datetime
 from stat import S_ISVTX, S_ISGID, S_ISUID, S_IXUSR, S_IWUSR, S_IRUSR, S_IXGRP
 from stat import S_IWGRP, S_IRGRP, S_IXOTH, S_IWOTH, S_IROTH, ST_MODE, S_ISDIR
 from stat import S_IFREG, ST_UID, ST_GID, S_ISREG, S_IFDIR, S_ISLNK, ST_MTIME
-
-import binascii, difflib, grp, os, pwd, string, logging, time
+import binascii
+import difflib
+import grp
+import logging
+import os
+import pwd
+import shutil
+import stat
+import string
+import time
 import Bcfg2.Client.Tools
+import Bcfg2.Options
+
+log = logging.getLogger('posix')
+
+# map between dev_type attribute and stat constants
+device_map = {'block': stat.S_IFBLK,
+              'char': stat.S_IFCHR,
+              'fifo': stat.S_IFIFO}
+
 
 def calcPerms(initial, perms):
     '''This compares ondisk permissions with specified ones'''
-    pdisp = [{1:S_ISVTX, 2:S_ISGID, 4:S_ISUID}, {1:S_IXUSR, 2:S_IWUSR, 4:S_IRUSR},
-             {1:S_IXGRP, 2:S_IWGRP, 4:S_IRGRP}, {1:S_IXOTH, 2:S_IWOTH, 4:S_IROTH}]
+    pdisp = [{1:S_ISVTX, 2:S_ISGID, 4:S_ISUID},
+             {1:S_IXUSR, 2:S_IWUSR, 4:S_IRUSR},
+             {1:S_IXGRP, 2:S_IWGRP, 4:S_IRGRP},
+             {1:S_IXOTH, 2:S_IWOTH, 4:S_IROTH}]
     tempperms = initial
     if len(perms) == 3:
         perms = '0%s' % (perms)
     pdigits = [int(perms[digit]) for digit in range(4)]
     for index in range(4):
-        for (num, perm) in pdisp[index].iteritems():
+        for (num, perm) in list(pdisp[index].items()):
             if pdigits[index] & num:
                 tempperms |= perm
     return tempperms
 
-log = logging.getLogger('posix')
-
 def normUid(entry):
-    '''This takes a user name or uid and returns the corrisponding uid or False'''
+    '''
+       This takes a user name or uid and
+       returns the corresponding uid or False
+    '''
     try:
         try:
             return int(entry.get('owner'))
@@ -36,14 +57,16 @@ def normUid(entry):
         return False
 
 def normGid(entry):
-    '''This takes a group name or gid and returns the corrisponding gid or False'''
+    '''
+       This takes a group name or gid and
+       returns the corresponding gid or False
+    '''
     try:
         try:
             return int(entry.get('group'))
         except:
             return int(grp.getgrnam(entry.get('group'))[2])
     except (OSError, KeyError):
-
         log.error('GID normalization failed for %s' % (entry.get('name')))
         return False
 
@@ -62,14 +85,37 @@ def isString(strng):
 
 class POSIX(Bcfg2.Client.Tools.Tool):
     '''POSIX File support code'''
-    __name__ = 'POSIX'
-    __handles__ = [('ConfigFile', None), ('Directory', None), ('Permissions', None), \
+    name = 'POSIX'
+    __handles__ = [('ConfigFile', None),
+                   ('Directory', None),
+                   ('Path', 'ConfigFile'),
+                   ('Path', 'Directory'),
+                   ('Path', 'HardLink'),
+                   ('Path', 'Permissions'),
+                   ('Path', 'SymLink'),
+                   ('Path', 'device'),
+                   ('Path', 'directory'),
+                   ('Path', 'file'),
+                   ('Path', 'hardlink'),
+                   ('Path', 'nonexistent'),
+                   ('Path', 'permissions'),
+                   ('Path', 'symlink'),
+                   ('Permissions', None),
                    ('SymLink', None)]
     __req__ = {'ConfigFile': ['name', 'owner', 'group', 'perms'],
                'Directory': ['name', 'owner', 'group', 'perms'],
+               'Path': ['name', 'type'],
                'Permissions': ['name', 'owner', 'group', 'perms'],
                'SymLink': ['name', 'to']}
 
+    # grab paranoid options from /etc/bcfg2.conf
+    opts = {'ppath': Bcfg2.Options.PARANOID_PATH,
+            'max_copies': Bcfg2.Options.PARANOID_MAX_COPIES}
+    setup = Bcfg2.Options.OptionParser(opts)
+    setup.parse([])
+    ppath = setup['ppath']
+    max_copies = setup['max_copies']
+
     def canInstall(self, entry):
         '''Check if entry is complete for installation'''
         if Bcfg2.Client.Tools.Tool.canInstall(self, entry):
@@ -101,19 +147,24 @@ class POSIX(Bcfg2.Client.Tools.Tool):
     def InstallSymLink(self, entry):
         '''Install SymLink Entry'''
         self.logger.info("Installing Symlink %s" % (entry.get('name')))
-        try:
-            fmode = os.lstat(entry.get('name'))[ST_MODE]
-            if S_ISREG(fmode) or S_ISLNK(fmode):
-                self.logger.debug("Non-directory entry already exists at %s" % \
-                                  (entry.get('name')))
-                os.unlink(entry.get('name'))
-            elif S_ISDIR(fmode):
-                self.logger.debug("Directory entry already exists at %s" % (entry.get('name')))
-                self.cmd.run("mv %s/ %s.bak" % (entry.get('name'), entry.get('name')))
-            else:
-                os.unlink(entry.get('name'))
-        except OSError:
-            self.logger.info("Symlink %s cleanup failed" % (entry.get('name')))
+        if os.path.lexists(entry.get('name')):
+            try:
+                fmode = os.lstat(entry.get('name'))[ST_MODE]
+                if S_ISREG(fmode) or S_ISLNK(fmode):
+                    self.logger.debug("Non-directory entry already exists at "
+                                      "%s. Unlinking entry." % \
+                                      (entry.get('name')))
+                    os.unlink(entry.get('name'))
+                elif S_ISDIR(fmode):
+                    self.logger.debug("Directory entry already exists at %s" %\
+                                      (entry.get('name')))
+                    self.cmd.run("mv %s/ %s.bak" % \
+                                 (entry.get('name'),
+                                  entry.get('name')))
+                else:
+                    os.unlink(entry.get('name'))
+            except OSError:
+                self.logger.info("Symlink %s cleanup failed" % (entry.get('name')))
         try:
             os.symlink(entry.get('to'), entry.get('name'))
             return True
@@ -135,7 +186,8 @@ class POSIX(Bcfg2.Client.Tools.Tool):
             owner = str(ondisk[ST_UID])
             group = str(ondisk[ST_GID])
         except (OSError, KeyError):
-            self.logger.error('User/Group resolution failed for path %s' % (entry.get('name')))
+            self.logger.error('User/Group resolution failed for path %s' % \
+                              entry.get('name'))
             owner = 'root'
             group = '0'
         finfo = os.stat(entry.get('name'))
@@ -159,7 +211,8 @@ class POSIX(Bcfg2.Client.Tools.Tool):
                 ex_ents = [e for e in entries if e not in modlist]
                 if ex_ents:
                     pruneTrue = False
-                    self.logger.debug("Directory %s contains extra entries:" % entry.get('name'))
+                    self.logger.debug("Directory %s contains extra entries:" % \
+                                      entry.get('name'))
                     self.logger.debug(ex_ents)
                     nqtext = entry.get('qtext', '') + '\n'
                     nqtext += "Directory %s contains extra entries:" % entry.get('name')
@@ -172,7 +225,8 @@ class POSIX(Bcfg2.Client.Tools.Tool):
         if not pTrue:
             if owner != str(normUid(entry)):
                 entry.set('current_owner', owner)
-                self.logger.debug("%s %s ownership wrong" % (entry.tag, entry.get('name')))
+                self.logger.debug("%s %s ownership wrong" % \
+                                  (entry.tag, entry.get('name')))
                 nqtext = entry.get('qtext', '') + '\n'
                 nqtext += "%s owner wrong. is %s should be %s" % \
                           (entry.get('name'), owner, entry.get('owner'))
@@ -206,7 +260,6 @@ class POSIX(Bcfg2.Client.Tools.Tool):
                 nnqtext += '\nInstall %s %s: (y/N) ' % (entry.tag, entry.get('name'))
                 entry.set('qtext', nnqtext)
         return pTrue and pruneTrue
-        
 
     def InstallDirectory(self, entry):
         '''Install Directory Entry'''
@@ -214,7 +267,8 @@ class POSIX(Bcfg2.Client.Tools.Tool):
         try:
             fmode = os.lstat(entry.get('name'))
             if not S_ISDIR(fmode[ST_MODE]):
-                self.logger.debug("Found a non-directory entry at %s" % (entry.get('name')))
+                self.logger.debug("Found a non-directory entry at %s" % \
+                                  (entry.get('name')))
                 try:
                     os.unlink(entry.get('name'))
                     exists = False
@@ -222,7 +276,8 @@ class POSIX(Bcfg2.Client.Tools.Tool):
                     self.logger.info("Failed to unlink %s" % (entry.get('name')))
                     return False
             else:
-                self.logger.debug("Found a pre-existing directory at %s" % (entry.get('name')))
+                self.logger.debug("Found a pre-existing directory at %s" % \
+                                  (entry.get('name')))
                 exists = True
         except OSError:
             # stat failed
@@ -235,7 +290,7 @@ class POSIX(Bcfg2.Client.Tools.Tool):
                     os.stat(parent)
                 except:
                     self.logger.debug('Creating parent path for directory %s' % (entry.get('name')))
-                    for idx in xrange(len(parent.split('/')[:-1])):
+                    for idx in range(len(parent.split('/')[:-1])):
                         current = '/'+'/'.join(parent.split('/')[1:2+idx])
                         try:
                             sloc = os.stat(current)
@@ -255,11 +310,15 @@ class POSIX(Bcfg2.Client.Tools.Tool):
             try:
                 os.mkdir(entry.get('name'))
             except OSError:
-                self.logger.error('Failed to create directory %s' % (entry.get('name')))
+                self.logger.error('Failed to create directory %s' % \
+                                  (entry.get('name')))
                 return False
         if entry.get('prune', 'false') == 'true' and entry.get("qtest"):
             for pname in entry.get("qtest").split(":"):
                 ulfailed = False
+                if os.path.isdir(pname):
+                    self.logger.info("Not removing extra directory %s, please check and remove manually" % pname)
+                    continue
                 try:
                     self.logger.debug("Unlinking file %s" % pname)
                     os.unlink(pname)
@@ -270,6 +329,50 @@ class POSIX(Bcfg2.Client.Tools.Tool):
                 return False
         return self.InstallPermissions(entry)
 
+    def VerifyhardLink(self, entry, _):
+        '''Verify HardLink Entry'''
+        try:
+            if os.path.samefile(entry.get('name'), entry.get('to')):
+                return True
+            self.logger.debug("Hardlink %s is incorrect" % \
+                              entry.get('name'))
+            entry.set('qtext', "Link %s to %s? [y/N] " % \
+                      (entry.get('name'),
+                       entry.get('to')))
+            return False
+        except OSError:
+            entry.set('current_exists', 'false')
+            entry.set('qtext', "Link %s to %s? [y/N] " % \
+                      (entry.get('name'),
+                       entry.get('to')))
+            return False
+
+    def InstallhardLink(self, entry):
+        '''Install HardLink Entry'''
+        self.logger.info("Installing Hardlink %s" % (entry.get('name')))
+        if os.path.lexists(entry.get('name')):
+            try:
+                fmode = os.lstat(entry.get('name'))[ST_MODE]
+                if S_ISREG(fmode) or S_ISLNK(fmode):
+                    self.logger.debug("Non-directory entry already exists at "
+                                      "%s. Unlinking entry." % (entry.get('name')))
+                    os.unlink(entry.get('name'))
+                elif S_ISDIR(fmode):
+                    self.logger.debug("Directory entry already exists at %s" % \
+                                      (entry.get('name')))
+                    self.cmd.run("mv %s/ %s.bak" % \
+                                 (entry.get('name'),
+                                  entry.get('name')))
+                else:
+                    os.unlink(entry.get('name'))
+            except OSError:
+                self.logger.info("Hardlink %s cleanup failed" % (entry.get('name')))
+        try:
+            os.link(entry.get('to'), entry.get('name'))
+            return True
+        except OSError:
+            return False
+
     def VerifyPermissions(self, entry, _):
         '''Verify Permissions entry'''
         return self.VerifyDirectory(entry, _)
@@ -281,7 +384,98 @@ class POSIX(Bcfg2.Client.Tools.Tool):
             os.chmod(entry.get('name'), calcPerms(S_IFDIR, entry.get('perms')))
             return True
         except (OSError, KeyError):
-            self.logger.error('Permission fixup failed for %s' % (entry.get('name')))
+            self.logger.error('Permission fixup failed for %s' % \
+                              (entry.get('name')))
+            return False
+
+    def Verifydevice(self, entry, _):
+        '''Verify device entry'''
+        try:
+            # check for file existence
+            filestat = os.stat(entry.get('name'))
+        except OSError:
+            entry.set('current_exists', 'false')
+            self.logger.debug("%s %s does not exist" %
+                              (entry.tag, entry.get('name')))
+            return False
+
+        try:
+            # attempt to verify device properties as specified in config
+            dev_type = entry.get('dev_type')
+            mode = calcPerms(device_map[dev_type],
+                             entry.get('mode', '0600'))
+            owner = entry.get('owner')
+            group = entry.get('group')
+            if dev_type in ['block', 'char']:
+                major = int(entry.get('major'))
+                minor = int(entry.get('minor'))
+                if major == os.major(filestat.st_rdev) and \
+                   minor == os.minor(filestat.st_rdev) and \
+                   mode == filestat.st_mode and \
+                   owner == filestat.st_uid and \
+                   group == filestat.st_gid:
+                    return True
+                else:
+                    return False
+            elif dev_type == 'fifo' and \
+                 mode == filestat.st_mode and \
+                 owner == filestat.st_uid and \
+                 group == filestat.st_gid:
+                return True
+            else:
+                self.logger.info('Device properties for %s incorrect' % \
+                                 entry.get('name'))
+                return False
+        except OSError:
+            self.logger.debug("%s %s failed to verify" %
+                              (entry.tag, entry.get('name')))
+            return False
+
+    def Installdevice(self, entry):
+        '''Install device entries'''
+        try:
+            # check for existing paths and remove them
+            filestat = os.lstat(entry.get('name'))
+            try:
+                os.unlink(entry.get('name'))
+                exists = False
+            except OSError:
+                self.logger.info('Failed to unlink %s' % \
+                                 entry.get('name'))
+                return False
+        except OSError:
+            exists = False
+
+        if not exists:
+            try:
+                dev_type = entry.get('dev_type')
+                mode = calcPerms(device_map[dev_type],
+                                 entry.get('mode', '0600'))
+                if dev_type in ['block', 'char']:
+                    major = int(entry.get('major'))
+                    minor = int(entry.get('minor'))
+                    device = os.makedev(major, minor)
+                    os.mknod(entry.get('name'), mode, device)
+                else:
+                    os.mknod(entry.get('name'), mode)
+                os.chown(entry.get('name'), normUid(entry), normGid(entry))
+                return True
+            except OSError:
+                self.logger.error('Failed to install %s' % entry.get('name'))
+                return False
+
+    def Verifynonexistent(self, entry, _):
+        '''Verify nonexistent entry'''
+        # return true if path does _not_ exist
+        return not os.path.lexists(entry.get('name'))
+
+    def Installnonexistent(self, entry):
+        '''Remove nonexistent entries'''
+        try:
+            os.remove(entry.get('name'))
+            return True
+        except OSError:
+            self.logger.error('Failed to remove %s' % entry.get('name'))
             return False
 
     def gatherCurrentData(self, entry):
@@ -317,18 +511,28 @@ class POSIX(Bcfg2.Client.Tools.Tool):
             tempdata = ''
         else:
             if entry.text == None:
-                self.logger.error("Cannot verify incomplete ConfigFile %s" % (entry.get('name')))
+                self.logger.error("Cannot verify incomplete ConfigFile %s" % \
+                                  (entry.get('name')))
                 return False
             tempdata = entry.text
             if type(tempdata) == unicode:
-                tempdata = tempdata.encode(self.setup['encoding'])
+                try:
+                    tempdata = tempdata.encode(self.setup['encoding'])
+                except UnicodeEncodeError, e:
+                    self.logger.error("Error encoding file %s:\n %s" % \
+                                      (entry.get('name'), e))
         try:
             content = open(entry.get('name')).read()
         except IOError, error:
-            self.logger.error("Failed to read %s: %s" % (error.filename, error.strerror))
-            return False
-        # comparison should be done with fingerprints or md5sum so it would be faster
-        # for big binary files
+            if error.strerror == "No such file or directory":
+                # print diff for files that don't exist (yet)
+                content = ''
+            else:
+                self.logger.error("Failed to read %s: %s" % \
+                                  (error.filename, error.strerror))
+                return False
+        # comparison should be done with fingerprints or
+        # md5sum so it would be faster for big binary files
         contentStatus = content == tempdata
         if not contentStatus:
             if tbin or not isString(content):
@@ -344,10 +548,12 @@ class POSIX(Bcfg2.Client.Tools.Tool):
                     now = time.time()
                     rawdiff.append(x)
                     if now - start > 5 and not longtime:
-                        self.logger.info("Diff of %s taking a long time" % (entry.get('name')))
+                        self.logger.info("Diff of %s taking a long time" % \
+                                         (entry.get('name')))
                         longtime = True
                     elif now - start > 30:
-                        self.logger.error("Diff of %s took too long; giving up" % (entry.get('name')))
+                        self.logger.error("Diff of %s took too long; giving up" % \
+                                          (entry.get('name')))
                         do_diff = False
                         break
                 if do_diff:
@@ -361,12 +567,12 @@ class POSIX(Bcfg2.Client.Tools.Tool):
                         eudiff = udiff.encode('ascii')
                     except:
                         eudiff = "Binary file: no diff printed"
- 
+
                     nqtext = entry.get('qtext', '')
 
                     if nqtext:
                         nqtext += '\n'
-                    nqtext += eudiff 
+                    nqtext += eudiff
                 else:
                     entry.set('current_bfile', binascii.b2a_base64(content))
                     nqtext = entry.get('qtext', '')
@@ -409,12 +615,34 @@ class POSIX(Bcfg2.Client.Tools.Tool):
                             return False
 
         # If we get here, then the parent directory should exist
-        if entry.get("paranoid", False) and self.setup.get("paranoid", False) \
-               and not (entry.get('current_exists', 'true') == 'false'):
+        if (entry.get("paranoid", False) == 'true') and \
+           self.setup.get("paranoid", False) and not \
+           (entry.get('current_exists', 'true') == 'false'):
             bkupnam = entry.get('name').replace('/', '_')
-            if self.cmd.run("cp %s /var/cache/bcfg2/%s" % (entry.get('name'), bkupnam))[0]:
+            # current list of backups for this ConfigFile
+            bkuplist = [f for f in os.listdir(self.ppath) if
+                              f.startswith(bkupnam)]
+            bkuplist.sort()
+            if len(bkuplist) == int(self.max_copies):
+                # remove the oldest backup available
+                oldest = bkuplist.pop(0)
+                self.logger.info("Removing %s" % oldest)
+                try:
+                    os.remove("%s/%s" % (self.ppath, oldest))
+                except:
+                    self.logger.error("Failed to remove %s/%s" % \
+                                      (self.ppath, oldest))
+                    return False
+            try:
+                # backup existing file
+                shutil.copy(entry.get('name'),
+                            "%s/%s_%s" % (self.ppath, bkupnam, datetime.now()))
+                self.logger.info("Backup of %s saved to %s" %
+                                 (entry.get('name'), self.ppath))
+            except IOError, e:
                 self.logger.error("Failed to create backup file for ConfigFile %s" % \
                                   (entry.get('name')))
+                self.logger.error(e)
                 return False
         try:
             newfile = open("%s.new"%(entry.get('name')), 'w')
@@ -451,5 +679,53 @@ class POSIX(Bcfg2.Client.Tools.Tool):
             if err.errno == 13:
                 self.logger.info("Failed to open %s for writing" % (entry.get('name')))
             else:
-                print err
+                print(err)
             return False
+
+    def Verifydirectory(self, entry, _):
+        ret = getattr(self, 'VerifyDirectory')
+        return ret(entry, _)
+
+    def Installdirectory(self, entry):
+        ret = getattr(self, 'InstallDirectory')
+        return ret(entry)
+
+    def Verifyfile(self, entry, _):
+        ret = getattr(self, 'VerifyConfigFile')
+        return ret(entry, _)
+
+    def Installfile(self, entry):
+        ret = getattr(self, 'InstallConfigFile')
+        return ret(entry)
+
+    def Verifynonexistent(self, entry, _):
+        # FIXME: not implemented
+        return True
+
+    def Installnonexistent(self, entry):
+        # FIXME: not implemented
+        return True
+
+    def Verifypermissions(self, entry, _):
+        ret = getattr(self, 'VerifyPermissions')
+        return ret(entry, _)
+
+    def Installpermissions(self, entry):
+        ret = getattr(self, 'InstallPermissions')
+        return ret(entry)
+
+    def Verifysymlink(self, entry, _):
+        ret = getattr(self, 'VerifySymLink')
+        return ret(entry, _)
+
+    def Installsymlink(self, entry):
+        ret = getattr(self, 'InstallSymLink')
+        return ret(entry)
+
+    def InstallPath(self, entry):
+        ret = getattr(self, 'Install%s' % entry.get('type'))
+        return ret(entry)
+
+    def VerifyPath(self, entry, _):
+        ret = getattr(self, 'Verify%s' % entry.get('type'))
+        return ret(entry, _)
