@@ -1,9 +1,14 @@
 '''Bcfg2 Support for RPMS'''
 
-__revision__ = '$Revision: 4923 $'
+__revision__ = '$Revision: 5651 $'
 
-import Bcfg2.Client.Tools, rpmtools, os.path, rpm, ConfigParser, sys
+import ConfigParser
+import os.path
+import rpm
+import rpmtools
+import Bcfg2.Client.Tools
 
+# Fix for python2.3
 try:
     set
 except NameError:
@@ -11,21 +16,21 @@ except NameError:
 
 class RPMng(Bcfg2.Client.Tools.PkgTool):
     '''Support for RPM packages'''
-    __name__ = 'RPMng'
+    name = 'RPMng'
 
     __execs__ = ['/bin/rpm', '/var/lib/rpm']
     __handles__ = [('Package', 'rpm')]
 
     __req__ = {'Package': ['name', 'version']}
     __ireq__ = {'Package': ['url']}
-    
+
     __new_req__ = {'Package': ['name'], 'Instance': ['version', 'release', 'arch']}
     __new_ireq__ = {'Package': ['uri'], \
                     'Instance': ['simplefile']}
-    
+
     __gpg_req__ = {'Package': ['name', 'version']}
     __gpg_ireq__ = {'Package': ['name', 'version']}
-    
+
     __new_gpg_req__ = {'Package': ['name'], 'Instance': ['version', 'release']}
     __new_gpg_ireq__ = {'Package': ['name'], 'Instance': ['version', 'release']}
 
@@ -48,14 +53,14 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
 
         # installonlypackages
         self.installOnlyPkgs = []
-        if RPMng_CP.has_option(self.__name__, 'installonlypackages'):
-            for i in RPMng_CP.get(self.__name__, 'installonlypackages').split(','):
+        if RPMng_CP.has_option(self.name, 'installonlypackages'):
+            for i in RPMng_CP.get(self.name, 'installonlypackages').split(','):
                 self.installOnlyPkgs.append(i.strip())
         if self.installOnlyPkgs == []:
             self.installOnlyPkgs = ['kernel', 'kernel-bigmem', 'kernel-enterprise', 'kernel-smp',
                                'kernel-modules', 'kernel-debug', 'kernel-unsupported',
                                'kernel-source', 'kernel-devel', 'kernel-default',
-                               'kernel-largesmp-devel', 'kernel-largesmp', 'kernel-xen', 
+                               'kernel-largesmp-devel', 'kernel-largesmp', 'kernel-xen',
                                'gpg-pubkey']
         if 'gpg-pubkey' not in self.installOnlyPkgs:
             self.installOnlyPkgs.append('gpg-pubkey')
@@ -63,45 +68,45 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
 
         # erase_flags
         self.erase_flags = []
-        if RPMng_CP.has_option(self.__name__, 'erase_flags'):
-            for i in RPMng_CP.get(self.__name__, 'erase_flags').split(','):
+        if RPMng_CP.has_option(self.name, 'erase_flags'):
+            for i in RPMng_CP.get(self.name, 'erase_flags').split(','):
                 self.erase_flags.append(i.strip())
         if self.erase_flags == []:
             self.erase_flags = ['allmatches']
         self.logger.debug('erase_flags = %s' % self.erase_flags)
 
         # pkg_checks
-        if RPMng_CP.has_option(self.__name__, 'pkg_checks'):
-            self.pkg_checks = RPMng_CP.get(self.__name__, 'pkg_checks').lower()
+        if RPMng_CP.has_option(self.name, 'pkg_checks'):
+            self.pkg_checks = RPMng_CP.get(self.name, 'pkg_checks').lower()
         else:
             self.pkg_checks = 'true'
         self.logger.debug('pkg_checks = %s' % self.pkg_checks)
 
         # pkg_verify
-        if RPMng_CP.has_option(self.__name__, 'pkg_verify'):
-            self.pkg_verify = RPMng_CP.get(self.__name__, 'pkg_verify').lower()
+        if RPMng_CP.has_option(self.name, 'pkg_verify'):
+            self.pkg_verify = RPMng_CP.get(self.name, 'pkg_verify').lower()
         else:
             self.pkg_verify = 'true'
         self.logger.debug('pkg_verify = %s' % self.pkg_verify)
 
         # installed_action
-        if RPMng_CP.has_option(self.__name__, 'installed_action'):
-            self.installed_action = RPMng_CP.get(self.__name__, 'installed_action').lower()
+        if RPMng_CP.has_option(self.name, 'installed_action'):
+            self.installed_action = RPMng_CP.get(self.name, 'installed_action').lower()
         else:
             self.installed_action = 'install'
         self.logger.debug('installed_action = %s' % self.installed_action)
 
         # version_fail_action
-        if RPMng_CP.has_option(self.__name__, 'version_fail_action'):
-            self.version_fail_action = RPMng_CP.get(self.__name__, 'version_fail_action').lower()
+        if RPMng_CP.has_option(self.name, 'version_fail_action'):
+            self.version_fail_action = RPMng_CP.get(self.name, 'version_fail_action').lower()
         else:
             self.version_fail_action = 'upgrade'
         self.logger.debug('version_fail_action = %s' % self.version_fail_action)
 
         # verify_fail_action
-        if self.__name__ == "RPMng":
-            if RPMng_CP.has_option(self.__name__, 'verify_fail_action'):
-                self.verify_fail_action = RPMng_CP.get(self.__name__, 'verify_fail_action').lower()
+        if self.name == "RPMng":
+            if RPMng_CP.has_option(self.name, 'verify_fail_action'):
+                self.verify_fail_action = RPMng_CP.get(self.name, 'verify_fail_action').lower()
             else:
                 self.verify_fail_action = 'reinstall'
         else: # yum can't reinstall packages.
@@ -109,13 +114,22 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
         self.logger.debug('verify_fail_action = %s' % self.verify_fail_action)
 
         # version_fail_action
-        if RPMng_CP.has_option(self.__name__, 'verify_flags'):
-            self.verify_flags = RPMng_CP.get(self.__name__, 'verify_flags').lower().split(',')
+        if RPMng_CP.has_option(self.name, 'verify_flags'):
+            self.verify_flags = RPMng_CP.get(self.name, 'verify_flags').lower().split(',')
         else:
             self.verify_flags = []
         if '' in self.verify_flags:
             self.verify_flags.remove('')
         self.logger.debug('version_fail_action = %s' % self.version_fail_action)
+        # Force a re- prelink of all packages if prelink exists.
+        # Many, if not most package verifies can be caused by out of date prelinking.
+        if os.path.isfile('/usr/sbin/prelink') and not self.setup['dryrun']:
+            cmdrc, output = self.cmd.run('/usr/sbin/prelink -a -mR')
+            if cmdrc == 0:
+                self.logger.debug('Pre-emptive prelink succeeded')
+            else:
+                # FIXME : this is dumb - what if the output is huge?
+                self.logger.error('Pre-emptive prelink failed: %s' % output)
 
 
     def RefreshPackages(self):
@@ -127,30 +141,30 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
 
             e.g.
 
-            self.installed['foo'] = [ {'name':'foo', 'epoch':None, 
-                                       'version':'1', 'release':2, 
+            self.installed['foo'] = [ {'name':'foo', 'epoch':None,
+                                       'version':'1', 'release':2,
                                        'arch':'i386'},
-                                      {'name':'foo', 'epoch':None, 
-                                       'version':'1', 'release':2, 
+                                      {'name':'foo', 'epoch':None,
+                                       'version':'1', 'release':2,
                                        'arch':'x86_64'} ]
         '''
         self.installed = {}
         refresh_ts = rpmtools.rpmtransactionset()
-        # Don't bother with signature checks at this stage. The GPG keys might 
+        # Don't bother with signature checks at this stage. The GPG keys might
         # not be installed.
         refresh_ts.setVSFlags(rpm._RPMVSF_NODIGESTS|rpm._RPMVSF_NOSIGNATURES)
         for nevra in rpmtools.rpmpackagelist(refresh_ts):
             self.installed.setdefault(nevra['name'], []).append(nevra)
         if self.setup['debug']:
-            print "The following package instances are installed:"
-            for name, instances in self.installed.iteritems():
+            print("The following package instances are installed:")
+            for name, instances in list(self.installed.items()):
                 self.logger.debug("    " + name)
                 for inst in instances:
-                    self.logger.debug("        %s" %self.str_evra(inst)) 
+                    self.logger.debug("        %s" %self.str_evra(inst))
         refresh_ts.closeDB()
         del refresh_ts
 
-    def VerifyPackage(self, entry, modlist):
+    def VerifyPackage(self, entry, modlist, pinned_version=None):
         '''
             Verify Package status for entry.
             Performs the following:
@@ -167,7 +181,7 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
                                          'verify_fail': True|False,
                                          'pkg': <Package Element Object>,
                                          'modlist': [ <filename>, ... ],
-                                         'verify' : [ <rpm --verify results> ]  
+                                         'verify' : [ <rpm --verify results> ]
                                        }, ......
                                   }
 
@@ -180,19 +194,29 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
         if instances == []:
             # We have an old style no Instance entry. Convert it to new style.
             instance = Bcfg2.Client.XML.SubElement(entry, 'Package')
-            for attrib in entry.attrib.keys():
+            for attrib in list(entry.attrib.keys()):
                 instance.attrib[attrib] = entry.attrib[attrib]
             if self.pkg_checks == 'true' and entry.get('pkg_checks', 'true') == 'true':
-                version, release = entry.get('version').split('-')
+                if 'any' in [entry.get('version'), pinned_version]:
+                    version, release = 'any', 'any'
+                elif entry.get('version') == 'auto':
+                    if pinned_version != None:
+                        version, release = pinned_version.split('-')
+                    else:
+                        return False
+                else:
+                    version, release = entry.get('version').split('-')
                 instance.set('version', version)
                 instance.set('release', release)
+                if entry.get('verify', 'true') == 'false':
+                    instance.set('verify', 'false')
             instances = [ instance ]
 
         self.logger.debug("Verifying package instances for %s" % entry.get('name'))
         package_fail = False
         qtext_versions = ''
 
-        if self.installed.has_key(entry.get('name')):
+        if entry.get('name') in self.installed:
             # There is at least one instance installed.
             if self.pkg_checks == 'true' and entry.get('pkg_checks', 'true') == 'true':
                 if entry.get('name') in self.installOnlyPkgs:
@@ -204,12 +228,15 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
                         self.instance_status[inst]['version_fail'] = False
                         if inst.tag == 'Package' and len(self.installed[entry.get('name')]) > 1:
                             self.logger.error("WARNING: Multiple instances of package %s are installed." % \
-                                                                             (entry.get('name')))
+                                              (entry.get('name')))
                         for pkg in self.installed[entry.get('name')]:
-                            if self.pkg_vr_equal(inst, pkg) or self.inst_evra_equal(inst, pkg):
+                            if inst.get('version') == 'any' or self.pkg_vr_equal(inst, pkg) \
+                               or self.inst_evra_equal(inst, pkg):
+                                if inst.get('version') == 'any':
+                                    self.logger.error("got any version")
                                 self.logger.debug("        %s" % self.str_evra(inst))
                                 self.instance_status[inst]['installed'] = True
-         
+
                                 if self.pkg_verify == 'true' and \
                                    inst.get('pkg_verify', 'true') == 'true':
                                     flags = inst.get('verify_flags', '').split(',') + self.verify_flags
@@ -220,24 +247,27 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
                                                            % (pkg.get('name'), self.str_evra(pkg), \
                                                               pkg.get('gpgkeyid', '')))
                                         self.logger.debug('         Disabling signature check.')
-        
+
                                     if self.setup.get('quick', False):
                                         if rpmtools.prelink_exists:
                                             flags += ['nomd5', 'nosize']
                                         else:
                                             flags += ['nomd5']
-                                    self.logger.debug("        verify_flags = %s" % flags) 
-        
-                                    vp_ts = rpmtools.rpmtransactionset()
-                                    self.instance_status[inst]['verify'] = \
-                                                           rpmtools.rpm_verify( vp_ts, pkg, flags)
-                                    vp_ts.closeDB()
-                                    del vp_ts
-    
+                                    self.logger.debug("        verify_flags = %s" % flags)
+
+                                    if inst.get('verify', 'true') == 'false':
+                                        self.instance_status[inst]['verify'] = None
+                                    else:
+                                        vp_ts = rpmtools.rpmtransactionset()
+                                        self.instance_status[inst]['verify'] = \
+                                                                             rpmtools.rpm_verify( vp_ts, pkg, flags)
+                                        vp_ts.closeDB()
+                                        del vp_ts
+
                         if self.instance_status[inst]['installed'] == False:
                             self.logger.info("        Package %s %s not installed." % \
                                          (entry.get('name'), self.str_evra(inst)))
-                                
+
                             qtext_versions = qtext_versions + 'I(%s) ' % self.str_evra(inst)
                             entry.set('current_exists', 'false')
                 else:
@@ -245,7 +275,7 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
                     for inst in instances:
                         self.instance_status.setdefault(inst, {})['installed'] = False
                         self.instance_status[inst]['version_fail'] = False
-    
+
                         # Only installed packages with the same architecture are
                         # relevant.
                         if inst.get('arch', None) == None:
@@ -253,7 +283,7 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
                         else:
                             arch_match = [pkg for pkg in self.installed[entry.get('name')] \
                                               if pkg.get('arch', None) == inst.get('arch', None)]
-    
+
                         if len(arch_match) > 1:
                             self.logger.error("Multiple instances of package %s installed with the same achitecture." % \
                                                   (entry.get('name')))
@@ -261,39 +291,44 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
                             # There is only one installed like there should be.
                             # Check that it is the right version.
                             for pkg in arch_match:
-                                if self.pkg_vr_equal(inst, pkg) or self.inst_evra_equal(inst, pkg):
+                                if inst.get('version') == 'any' or self.pkg_vr_equal(inst, pkg) or \
+                                       self.inst_evra_equal(inst, pkg):
                                     self.logger.debug("        %s" % self.str_evra(inst))
                                     self.instance_status[inst]['installed'] = True
-    
+
                                     if self.pkg_verify == 'true' and \
                                        inst.get('pkg_verify', 'true') == 'true':
                                         flags = inst.get('verify_flags', '').split(',') + self.verify_flags
-                                        if pkg.get('gpgkeyid', '')[-8:] not in self.gpg_keyids:
+                                        if pkg.get('gpgkeyid', '')[-8:] not in self.gpg_keyids and \
+                                           'nosignature' not in flags:
                                             flags += ['nosignature', 'nodigest']
                                             self.logger.info('WARNING: Package %s %s requires GPG Public key with ID %s'\
                                                          % (pkg.get('name'), self.str_evra(pkg), \
                                                             pkg.get('gpgkeyid', '')))
                                             self.logger.info('         Disabling signature check.')
-        
+
                                         if self.setup.get('quick', False):
                                             if rpmtools.prelink_exists:
                                                 flags += ['nomd5', 'nosize']
                                             else:
                                                 flags += ['nomd5']
-                                        self.logger.debug("        verify_flags = %s" % flags) 
-        
-                                        vp_ts = rpmtools.rpmtransactionset()
-                                        self.instance_status[inst]['verify'] = \
-                                                         rpmtools.rpm_verify( vp_ts, pkg, flags )
-                                        vp_ts.closeDB()
-                                        del vp_ts
-    
+                                        self.logger.debug("        verify_flags = %s" % flags)
+
+                                        if inst.get('verify', 'true') == 'false':
+                                            self.instance_status[inst]['verify'] = None
+                                        else:
+                                            vp_ts = rpmtools.rpmtransactionset()
+                                            self.instance_status[inst]['verify'] = \
+                                                                                 rpmtools.rpm_verify( vp_ts, pkg, flags )
+                                            vp_ts.closeDB()
+                                            del vp_ts
+
                                 else:
                                     # Wrong version installed.
                                     self.instance_status[inst]['version_fail'] = True
                                     self.logger.info("        Wrong version installed.  Want %s, but have %s"\
                                                     % (self.str_evra(inst), self.str_evra(pkg)))
-                            
+
                                     qtext_versions = qtext_versions + 'U(%s -> %s) ' % \
                                                           (self.str_evra(pkg), self.str_evra(inst))
                         elif len(arch_match) == 0:
@@ -301,60 +336,59 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
                             self.instance_status[inst]['installed'] = False
                             self.logger.info("        %s is not installed." % self.str_evra(inst))
                             qtext_versions = qtext_versions + 'I(%s) ' % self.str_evra(inst)
-    
+
                 # Check the rpm verify results.
                 for inst in instances:
                     instance_fail = False
-                    # Dump the rpm verify results. 
+                    # Dump the rpm verify results.
                     #****Write something to format this nicely.*****
                     if self.setup['debug'] and self.instance_status[inst].get('verify', None):
                         self.logger.debug(self.instance_status[inst]['verify'])
-    
+
                     self.instance_status[inst]['verify_fail'] = False
                     if self.instance_status[inst].get('verify', None):
                         if len(self.instance_status[inst].get('verify')) > 1:
                             self.logger.info("WARNING: Verification of more than one package instance.")
- 
+
                         for result in self.instance_status[inst]['verify']:
-    
+
                             # Check header results
                             if result.get('hdr', None):
                                 instance_fail = True
                                 self.instance_status[inst]['verify_fail'] = True
-        
+
                             # Check dependency results
                             if result.get('deps', None):
                                 instance_fail = True
                                 self.instance_status[inst]['verify_fail'] = True
-                             
+
                             # Check the rpm verify file results against the modlist
-                            # and per Instance Ignores.
+                            # and entry and per Instance Ignores.
+                            ignores = [ig.get('name') for ig in entry.findall('Ignore')] + [ig.get('name') for ig in inst.findall('Ignore')]
                             for file_result in result.get('files', []):
-                                if file_result[-1] not in modlist and \
-                                   file_result[-1] not in \
-                                         [ignore.get('name') for ignore in inst.findall('Ignore')]:
+                                if file_result[-1] not in modlist + ignores:
                                     instance_fail = True
                                     self.instance_status[inst]['verify_fail'] = True
                                 else:
                                     self.logger.debug("        Modlist/Ignore match: %s" % \
                                                                                  (file_result[-1]))
-    
+
                         if instance_fail == True:
                             self.logger.debug("*** Instance %s failed RPM verification ***" % \
-                                                                           self.str_evra(inst))
+                                              self.str_evra(inst))
                             qtext_versions = qtext_versions + 'R(%s) ' % self.str_evra(inst)
                             self.modlists[entry] = modlist
 
                             # Attach status structure for return to server for reporting.
                             inst.set('verify_status', str(self.instance_status[inst]))
-    
+
                     if self.instance_status[inst]['installed'] == False or \
                        self.instance_status[inst].get('version_fail', False)== True or \
                        self.instance_status[inst].get('verify_fail', False) == True:
                         package_fail = True
                         self.instance_status[inst]['pkg'] = entry
                         self.modlists[entry] = modlist
-    
+
                 # Find Installed Instances that are not in the Config.
                 extra_installed = self.FindExtraInstances(entry, self.installed[entry.get('name')])
                 if extra_installed != None:
@@ -363,25 +397,25 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
                     for inst in extra_installed.findall('Instance'):
                         qtext_versions = qtext_versions + 'D(%s) ' % self.str_evra(inst)
                     self.logger.debug("Found Extra Instances %s" % qtext_versions)
-               
+
                 if package_fail == True:
                     self.logger.info("        Package %s failed verification." % \
                                                               (entry.get('name')))
                     qtext = 'Install/Upgrade/delete Package %s instance(s) - %s (y/N) ' % \
                                                   (entry.get('name'), qtext_versions)
                     entry.set('qtext', qtext)
-    
+
                     bcfg2_versions = ''
                     for bcfg2_inst in [inst for inst in instances if inst.tag == 'Instance']:
                         bcfg2_versions = bcfg2_versions + '(%s) ' % self.str_evra(bcfg2_inst)
                     if bcfg2_versions != '':
                         entry.set('version', bcfg2_versions)
                     installed_versions = ''
-    
+
                     for installed_inst in self.installed[entry.get('name')]:
                         installed_versions = installed_versions + '(%s) ' % \
                                                                       self.str_evra(installed_inst)
-    
+
                     entry.set('current_version', installed_versions)
                     return False
 
@@ -400,7 +434,7 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
             if bcfg2_versions != '':
                 entry.set('version', bcfg2_versions)
             entry.set('qtext', "Install Package %s Instance(s) %s? (y/N) " % \
-                                                        (entry.get('name'), qtext_versions))
+                      (entry.get('name'), qtext_versions))
 
             return False
         return True
@@ -409,7 +443,7 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
         '''
            Remove specified entries.
 
-           packages is a list of Package Entries with Instances generated 
+           packages is a list of Package Entries with Instances generated
            by FindExtraPackages().
         '''
         self.logger.debug('Running RPMng.RemovePackages()')
@@ -433,7 +467,7 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
                     self.logger.info("         This package will be deleted in a future version of the RPMng driver.")
                 #pkgspec_list.append(pkg_spec)
 
-        erase_results = rpmtools.rpm_erase(pkgspec_list, self.erase_flags) 
+        erase_results = rpmtools.rpm_erase(pkgspec_list, self.erase_flags)
         if erase_results == []:
             self.modified += packages
             for pkg in pkgspec_list:
@@ -461,7 +495,7 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
                                                    % (pkgspec.get('name'), self.str_evra(pkgspec)))
                         self.logger.info("         This package will be deleted in a future version of the RPMng driver.")
                         continue # Don't delete the gpg-pubkey packages for now.
-                    erase_results = rpmtools.rpm_erase([pkgspec], self.erase_flags) 
+                    erase_results = rpmtools.rpm_erase([pkgspec], self.erase_flags)
                     if erase_results == []:
                         pkg_modified = True
                         self.logger.info("Deleted %s %s" % \
@@ -478,7 +512,7 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
 
     def FixInstance(self, instance, inst_status):
         '''
-           Control if a reinstall of a package happens or not based on the 
+           Control if a reinstall of a package happens or not based on the
            results from RPMng.VerifyPackage().
 
            Return True to reinstall, False to not reintstall.
@@ -491,8 +525,8 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
                 fix = True
             else:
                 self.logger.debug('Installed Action for %s %s is to not install' % \
-                                                     (inst_status.get('pkg').get('name'),
-                                                      self.str_evra(instance)))
+                                  (inst_status.get('pkg').get('name'),
+                                   self.str_evra(instance)))
 
         elif inst_status.get('version_fail', False) == True:
             if instance.get('version_fail_action', 'upgrade') == "upgrade" and \
@@ -500,22 +534,22 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
                 fix = True
             else:
                 self.logger.debug('Version Fail Action for %s %s is to not upgrade' % \
-                                                     (inst_status.get('pkg').get('name'),
-                                                      self.str_evra(instance)))
+                                  (inst_status.get('pkg').get('name'),
+                                   self.str_evra(instance)))
 
-        elif inst_status.get('verify_fail', False) == True and self.__name__ == "RPMng":
+        elif inst_status.get('verify_fail', False) == True and self.name == "RPMng":
             # yum can't reinstall packages so only do this for rpm.
             if instance.get('verify_fail_action', 'reinstall') == "reinstall" and \
                self.verify_fail_action == "reinstall":
                 for inst in inst_status.get('verify'):
-                    # This needs to be a for loop rather than a straight get() 
+                    # This needs to be a for loop rather than a straight get()
                     # because the underlying routines handle multiple packages
                     # and return a list of results.
                     self.logger.debug('reinstall_check: %s %s:%s-%s.%s' % inst.get('nevra'))
 
                     if inst.get("hdr", False):
                         fix = True
-    
+
                     elif inst.get('files', False):
                         # Parse rpm verify file results
                         for file_result in inst.get('files', []):
@@ -533,24 +567,24 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
                                                       self.str_evra(instance)))
 
         return fix
-    
+
     def Install(self, packages, states):
         '''
-           Try and fix everything that RPMng.VerifyPackages() found wrong for 
-           each Package Entry.  This can result in individual RPMs being 
-           installed (for the first time), reinstalled, deleted, downgraded 
+           Try and fix everything that RPMng.VerifyPackages() found wrong for
+           each Package Entry.  This can result in individual RPMs being
+           installed (for the first time), reinstalled, deleted, downgraded
            or upgraded.
 
-           packages is a list of Package Elements that has 
+           packages is a list of Package Elements that has
                states[<Package Element>] == False
 
            The following effects occur:
            - states{} is conditionally updated for each package.
            - self.installed{} is rebuilt, possibly multiple times.
-           - self.instance_statusi{} is conditionally updated for each instance 
+           - self.instance_statusi{} is conditionally updated for each instance
              of a package.
-           - Each package will be added to self.modified[] if its states{} 
-             entry is set to True. 
+           - Each package will be added to self.modified[] if its states{}
+             entry is set to True.
         '''
         self.logger.info('Runing RPMng.Install()')
 
@@ -692,6 +726,12 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
         if not self.handlesEntry(entry):
             return False
 
+        if 'failure' in entry.attrib:
+            self.logger.error("Cannot install entry %s:%s with bind failure" % \
+                              (entry.tag, entry.get('name')))
+            return False
+
+
         instances = entry.findall('Instance')
 
         # If the entry wasn't verifiable, then we really don't want to try and fix something
@@ -756,20 +796,26 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
     def canVerify(self, entry):
         '''
             Test if entry has enough information to be verified.
- 
+
             Three types of entries are checked.
                Old style Package
                New style Package with Instances
                pgp-pubkey packages
 
-           Also the old style entries get modified after the first 
+           Also the old style entries get modified after the first
            VerifyPackage() run, so there needs to be a second test.
         '''
         if not self.handlesEntry(entry):
             return False
 
+        if 'failure' in entry.attrib:
+            self.logger.error("Entry %s:%s reports bind failure: %s" % \
+                              (entry.tag, entry.get('name'), entry.get('failure')))
+            return False
+
         # We don't want to do any checks so we don't care what the entry has in it.
-        if self.pkg_checks == 'false' or entry.get('pkg_checks', 'true').lower() == 'false':
+        if self.pkg_checks == 'false' or \
+               entry.get('pkg_checks', 'true').lower() == 'false':
             return True
 
         instances = entry.findall('Instance')
@@ -777,7 +823,7 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
         if not instances:
             # Old non Instance format, unmodified.
             if entry.get('name') == 'gpg-pubkey':
-                # gpg-pubkey packages aren't really pacakges, so we have to do 
+                # gpg-pubkey packages aren't really pacakges, so we have to do
                 # something a little different.
                 # Check that the Package Level has what we need for verification.
                 if [attr for attr in self.__gpg_req__[entry.tag] if attr not in entry.attrib]:
@@ -791,7 +837,7 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
                     return False
         else:
             if entry.get('name') == 'gpg-pubkey':
-                # gpg-pubkey packages aren't really pacakges, so we have to do 
+                # gpg-pubkey packages aren't really pacakges, so we have to do
                 # something a little different.
                 # Check that the Package Level has what we need for verification.
                 if [attr for attr in self.__new_gpg_req__[entry.tag] if attr not in entry.attrib]:
@@ -829,12 +875,13 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
         packages = [entry.get('name') for entry in self.getSupportedEntries()]
         extras = []
 
-        for (name, instances) in self.installed.iteritems():
+        for (name, instances) in list(self.installed.items()):
             if name not in packages:
                 extra_entry = Bcfg2.Client.XML.Element('Package', name=name, type=self.pkgtype)
                 for installed_inst in instances:
-                    self.logger.info("Extra Package %s %s." % \
-                                               (name, self.str_evra(installed_inst)))
+                    if self.setup['extra']:
+                        self.logger.info("Extra Package %s %s." % \
+                                         (name, self.str_evra(installed_inst)))
                     tmp_entry = Bcfg2.Client.XML.SubElement(extra_entry, 'Instance', \
                                      version = installed_inst.get('version'), \
                                      release = installed_inst.get('release'))
@@ -866,7 +913,7 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
                 if not_found == True:
                     # Extra package.
                     self.logger.info("Extra InstallOnlyPackage %s %s." % \
-                                               (name, self.str_evra(installed_inst)))
+                                     (name, self.str_evra(installed_inst)))
                     tmp_entry = Bcfg2.Client.XML.SubElement(extra_entry, 'Instance', \
                                      version = installed_inst.get('version'), \
                                      release = installed_inst.get('release'))
@@ -885,7 +932,7 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
                         break
                 if not_found:
                     self.logger.info("Extra Normal Package Instance %s %s" % \
-                                                        (name, self.str_evra(installed_inst)))
+                                     (name, self.str_evra(installed_inst)))
                     tmp_entry = Bcfg2.Client.XML.SubElement(extra_entry, 'Instance', \
                                      version = installed_inst.get('version'), \
                                      release = installed_inst.get('release'))
@@ -904,14 +951,14 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
             Convert evra dict entries to a string.
         '''
         if instance.get('epoch', '*') in ['*', None]:
-            return '%s-%s.%s' % (instance.get('version', '*'), 
+            return '%s-%s.%s' % (instance.get('version', '*'),
                                  instance.get('release', '*'),
-                                 instance.get('arch', '*')) 
-        else: 
+                                 instance.get('arch', '*'))
+        else:
             return '%s:%s-%s.%s' % (instance.get('epoch', '*'),
-                                    instance.get('version', '*'), 
+                                    instance.get('version', '*'),
                                     instance.get('release', '*'),
-                                    instance.get('arch', '*')) 
+                                    instance.get('arch', '*'))
 
     def pkg_vr_equal(self, config_entry, installed_entry):
         '''
@@ -947,12 +994,12 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
             return False
 
     def getinstalledgpg(self):
-        ''' 
+        '''
            Create a list of installed GPG key IDs.
 
            The pgp-pubkey package version is the least significant 4 bytes
            (big-endian) of the key ID which is good enough for our purposes.
-        ''' 
+        '''
         init_ts = rpmtools.rpmtransactionset()
         init_ts.setVSFlags(rpm._RPMVSF_NODIGESTS|rpm._RPMVSF_NOSIGNATURES)
         gpg_hdrs = rpmtools.getheadersbykeyword(init_ts, **{'name':'gpg-pubkey'})
