@@ -1,6 +1,6 @@
 # This is the bcfg2 support for solaris sysv packages
 '''This provides bcfg2 support for Solaris SYSV packages'''
-__revision__ = '$Revision: 4703 $'
+__revision__ = '$Revision: 5483 $'
 
 import tempfile, Bcfg2.Client.Tools, Bcfg2.Client.XML
 
@@ -25,15 +25,19 @@ class SYSV(Bcfg2.Client.Tools.PkgTool):
     __handles__ = [('Package', 'sysv')]
     __req__ = {'Package': ['name', 'version']}
     __ireq__ = {'Package': ['name', 'url', 'version']}
-    __name__ = 'SYSV'
+    name = 'SYSV'
     pkgtype = 'sysv'
     pkgtool = ("/usr/sbin/pkgadd %s -n -d %%s", (('%s %s', ['url', 'name'])))
 
     def __init__(self, logger, setup, config):
         Bcfg2.Client.Tools.PkgTool.__init__(self, logger, setup, config)
-        self.noaskname = tempfile.mktemp()
+        # noaskfile needs to live beyond __init__ otherwise file is removed
+        self.noaskfile = tempfile.NamedTemporaryFile()
+        self.noaskname = self.noaskfile.name
         try:
-            open(self.noaskname, 'w+').write(noask)
+            self.noaskfile.write(noask)
+            # flush admin file contents to disk
+            self.noaskfile.flush()
             self.pkgtool = (self.pkgtool[0] % ("-a %s" % (self.noaskname)), \
                             self.pkgtool[1])
         except:
@@ -45,7 +49,12 @@ class SYSV(Bcfg2.Client.Tools.PkgTool):
         # Build list of packages
         lines = self.cmd.run("/usr/bin/pkginfo -x")[1]
         while lines:
-            version = lines.pop().split()[1]
+	    # Splitting on whitespace means that packages with spaces in
+	    # their version numbers don't work right.  Found this with
+	    # IBM TSM software with package versions like
+	    #		"Version 6 Release 1 Level 0.0"
+	    # Should probably be done with a regex but this works.
+            version = lines.pop().split(') ')[1]
             pkg = lines.pop().split()[0]
             self.installed[pkg] = version
 
@@ -54,14 +63,19 @@ class SYSV(Bcfg2.Client.Tools.PkgTool):
         if not entry.get('version'):
             self.logger.info("Insufficient information of Package %s; cannot Verify" % entry.get('name'))
             return False
+
+        desiredVersion = entry.get('version')
+        if desiredVersion == 'any':
+            desiredVersion = self.installed.get(entry.get('name'), desiredVersion)
+
         cmdrc = self.cmd.run("/usr/bin/pkginfo -q -v \"%s\" %s" % \
-                             (entry.get('version'), entry.get('name')))[0]
+                             (desiredVersion, entry.get('name')))[0]
 
         if cmdrc != 0:
             if entry.get('name') in self.installed:
                 self.logger.debug("Package %s version incorrect: have %s want %s" \
                                   % (entry.get('name'), self.installed[entry.get('name')],
-                                     entry.get('version')))
+                                     desiredVersion))
             else:
                 self.logger.debug("Package %s not installed" % (entry.get("name")))
         else:
