@@ -1,15 +1,24 @@
 '''This contains all Bcfg2 Tool modules'''
-__revision__ = '$Revision: 4923 $'
+__revision__ = '$Revision: 5445 $'
 
-__all__ = ["Action", "APT", "Blast", "Chkconfig", "DebInit", "Encap",
+__all__ = ["Action", "APT", "Blast", "Chkconfig", "DebInit", "Encap", "IPS",
            "FreeBSDInit", "FreeBSDPackage", "launchd", "Portage", "POSIX",
-           "RPM", "RPMng", 'rpmtools', "RcUpdate", "SMF", "SYSV", "Yum",
-           "YUMng"]
+           "RPMng", 'rpmtools', "RcUpdate", "SMF", "SYSV", "YUMng"]
 
 drivers = [item for item in __all__ if item not in ['rpmtools']]
 default = [item for item in drivers if item not in ['RPM', 'Yum']]
 
-import os, popen2, stat, sys, Bcfg2.Client.XML, time
+# suppress popen2 warnings for python 2.3
+import warnings
+warnings.filterwarnings("ignore", "The popen2 module is deprecated.*",
+                        DeprecationWarning)
+import os
+import popen2
+import stat
+import sys
+import time
+
+import Bcfg2.Client.XML
 
 class toolInstantiationError(Exception):
     '''This error is called if the toolset cannot be instantiated'''
@@ -66,7 +75,7 @@ class Tool:
     '''
     All tools subclass this. It defines all interfaces that need to be defined
     '''
-    __name__ = 'Tool'
+    name = 'Tool'
     __execs__ = []
     __handles__ = []
     __req__ = {}
@@ -88,7 +97,7 @@ class Tool:
                 mode = stat.S_IMODE(os.stat(filename)[stat.ST_MODE])
                 if mode & stat.S_IEXEC != stat.S_IEXEC:
                     self.logger.debug("%s: %s not executable" % \
-                                      (self.__name__, filename))
+                                      (self.name, filename))
                     raise toolInstantiationError
             except OSError:
                 raise toolInstantiationError
@@ -108,13 +117,12 @@ class Tool:
         '''Dispatch verify calls to underlying methods'''
         if not structures:
             structures = self.config.getchildren()
+        mods = self.buildModlist()
         for (struct, entry) in [(struct, entry) for struct in structures \
                                 for entry in struct.getchildren() \
                                 if self.canVerify(entry)]:
             try:
                 func = getattr(self, "Verify%s" % (entry.tag))
-                mods = self.buildModlist(entry, struct)
-                mods += [c.get('name') for c in entry.findall("Ignore")]
                 states[entry] = func(entry, mods)
             except:
                 self.logger.error(
@@ -139,19 +147,21 @@ class Tool:
 
     def getSupportedEntries(self):
         '''return a list of supported entries'''
-        return [entry for struct in self.config.getchildren() for entry in struct.getchildren() \
+        return [entry for struct in \
+                self.config.getchildren() for entry in \
+                struct.getchildren() \
                 if self.handlesEntry(entry)]
 
     def handlesEntry(self, entry):
         '''return if entry is handled by this Tool'''
         return (entry.tag, entry.get('type')) in self.__handles__
 
-    def buildModlist(self, entry, struct):
+    def buildModlist(self):
         '''Build a list of potentially modified POSIX paths for this entry'''
-        if struct.tag != 'Bundle':
-            return []
-        return [sentry.get('name') for sentry in struct if sentry.tag in \
-                ['ConfigFile', 'SymLink', 'Directory', 'Permissions']]
+        return [entry.get('name') for struct in self.config.getchildren() \
+                for entry in struct.getchildren() \
+                if entry.tag in ['ConfigFile', 'SymLink', 'Directory',
+                                 'Permissions', 'Ignore', 'Path']]
 
     def gatherCurrentData(self, entry):
         '''Default implementation of the information gathering routines'''
@@ -162,6 +172,11 @@ class Tool:
         if not self.handlesEntry(entry):
             return False
 
+        if 'failure' in entry.attrib:
+            self.logger.error("Entry %s:%s reports bind failure: %s" % \
+                              (entry.tag, entry.get('name'), entry.get('failure')))
+            return False
+
         missing = [attr for attr in self.__req__[entry.tag] \
                    if attr not in entry.attrib]
         if missing:
@@ -186,6 +201,11 @@ class Tool:
         if not self.handlesEntry(entry):
             return False
 
+        if 'failure' in entry.attrib:
+            self.logger.error("Cannot install entry %s:%s with bind failure" % \
+                              (entry.tag, entry.get('name')))
+            return False
+
         missing = [attr for attr in self.__ireq__[entry.tag] \
                    if attr not in entry.attrib or not entry.attrib[attr]]
         if missing:
@@ -197,10 +217,13 @@ class Tool:
         return True
 
 class PkgTool(Tool):
-    '''PkgTool provides a one-pass install with fallback for use with packaging systems'''
+    '''
+       PkgTool provides a one-pass install with
+       fallback for use with packaging systems
+    '''
     pkgtool = ('echo %s', ('%s', ['name']))
     pkgtype = 'echo'
-    __name__ = 'PkgTool'
+    name = 'PkgTool'
 
     def __init__(self, logger, setup, config):
         Tool.__init__(self, logger, setup, config)
@@ -214,7 +237,10 @@ class PkgTool(Tool):
         return False
 
     def Install(self, packages, states):
-        '''Run a one-pass install, followed by single pkg installs in case of failure'''
+        '''
+           Run a one-pass install, followed by
+           single pkg installs in case of failure
+        '''
         self.logger.info("Trying single pass package install for pkgtype %s" % \
                          self.pkgtype)
 
@@ -229,7 +255,7 @@ class PkgTool(Tool):
             self.logger.info("Single Pass Succeded")
             # set all package states to true and flush workqueues
             pkgnames = [pkg.get('name') for pkg in packages]
-            for entry in [entry for entry in states.keys()
+            for entry in [entry for entry in list(states.keys())
                           if entry.tag == 'Package'
                           and entry.get('type') == self.pkgtype
                           and entry.get('name') in pkgnames]:
@@ -270,26 +296,60 @@ class PkgTool(Tool):
     def FindExtraPackages(self):
         '''Find extra packages'''
         packages = [entry.get('name') for entry in self.getSupportedEntries()]
-        extras = [data for data in self.installed.iteritems() if data[0] not in packages]
+        extras = [data for data in list(self.installed.items()) \
+                  if data[0] not in packages]
         return [Bcfg2.Client.XML.Element('Package', name=name, \
                                          type=self.pkgtype, version=version) \
                                          for (name, version) in extras]
 
 class SvcTool(Tool):
     '''This class defines basic Service behavior'''
-    __name__ = 'SvcTool'
-    __svcrestart__ = 'reload'
+    name = 'SvcTool'
+
+    def get_svc_command(self, service, action):
+        '''Return the basename of the command used to start/stop services'''
+        return '/etc/init.d/%s %s' % (service.get('name'), action)
+
+    def start_service(self, service):
+        self.logger.debug('Starting service %s' % service.get('name'))
+        return self.cmd.run(self.get_svc_command(service, 'start'))[0]
+
+    def stop_service(self, service):
+        self.logger.debug('Stopping service %s' % service.get('name'))
+        return self.cmd.run(self.get_svc_command(service, 'stop'))[0]
+
+    def restart_service(self, service):
+        self.logger.debug('Restarting service %s' % service.get('name'))
+        restart_target = 'restart'
+        if service.get('mode', 'default') == 'custom':
+            restart_target = service.get('target', 'restart')
+        return self.cmd.run(self.get_svc_command(service, restart_target))[0]
+
+    def check_service(self, service):
+        # not supported for this driver
+        return 0
 
     def BundleUpdated(self, bundle, states):
         '''The Bundle has been updated'''
-        for entry in bundle:
-            if self.handlesEntry(entry):
-                if entry.get('status') == 'on' and not self.setup['build']:
-                    self.logger.debug('Restarting service %s' % entry.get('name'))
-                    rc = self.cmd.run('/etc/init.d/%s %s' % \
-                                      (entry.get('name'), entry.get('reload', self.__svcrestart__)))[0]
+        if self.setup['servicemode'] == 'disabled':
+            return
+
+        for entry in [ent for ent in bundle if self.handlesEntry(ent)]:
+            if entry.get('mode', 'default') == 'manual':
+                continue
+            # need to handle servicemode = (build|default)
+            # need to handle mode = (default|supervised|custom)
+            if entry.get('status') == 'on':
+                if self.setup['servicemode'] == 'build':
+                    rc = self.stop_service(entry)
                 else:
-                    self.logger.debug('Stopping service %s' % entry.get('name'))
-                    rc = self.cmd.run('/etc/init.d/%s stop' %  (entry.get('name')))[0]
-                if rc:
-                    self.logger.error("Failed to restart service %s" % (entry.get('name')))
+                    if self.setup['interactive']:
+                        prompt = 'Restart service %s?: (y/N): ' % entry.get('name')
+                        if raw_input(prompt) not in ['y', 'Y']:
+                            continue
+                    rc = self.restart_service(entry)
+            else:
+                rc = self.stop_service(entry)
+            if rc:
+                self.logger.error("Failed to manipulate service %s" % \
+                                  (entry.get('name')))
