@@ -1,5 +1,5 @@
 '''XML lib compatibility layer for the Bcfg2 client'''
-__revision__ = '$Revision: 4835 $'
+__revision__ = '$Revision: 5178 $'
 
 # library will use lxml, then builtin xml.etree, then ElementTree
 
@@ -30,8 +30,8 @@ except ImportError:
                 return elementtree.ElementTree.tostring(e)
 
         except ImportError:
-            print "Failed to load lxml, xml.etree and elementtree.ElementTree"
-            print "Cannot continue"
-            raise SystemExit, 1
+            print("Failed to load lxml, xml.etree and elementtree.ElementTree")
+            print("Cannot continue")
+            raise SystemExit(1)
 
 len([Element, SubElement, XML, tostring, ParseError])
