@@ -1,7 +1,11 @@
 '''Option parsing library for utilities'''
-__revision__ = '$Revision: 4973 $'
+__revision__ = '$Revision: 5482 $'
 
-import getopt, os, socket, sys, ConfigParser, Bcfg2.Client.Tools
+import ConfigParser
+import getopt
+import os
+import sys
+import Bcfg2.Client.Tools
 
 def bool_cook(x):
     if x:
@@ -18,6 +22,7 @@ DEFAULT_INSTALL_PREFIX = '/usr' #/usr
 class Option(object):
     cfpath = DEFAULT_CONFIG_LOCATION
     __cfp = False
+
     def getCFP(self):
         if not self.__cfp:
             self.__cfp = ConfigParser.ConfigParser()
@@ -32,7 +37,7 @@ class Option(object):
             return self.cook(value)
         else:
             return value
-    
+
     def __init__(self, desc, default, cmd=False, odesc=False,
                  env=False, cf=False, cook=False, long_arg=False):
         self.desc = desc
@@ -62,11 +67,11 @@ class Option(object):
                 msg = self.cmd
             if self.odesc:
                 if self.long:
-                    msg = "%-27s" % ("%s=%s" % (self.cmd, self.odesc))
+                    msg = "%-28s" % ("%s=%s" % (self.cmd, self.odesc))
                 else:
-                    msg += '%-24s' % (self.odesc)
+                    msg += '%-25s' % (self.odesc)
             else:
-                msg += '%-24s' % ('')
+                msg += '%-25s' % ('')
             msg += "%s\n" % self.desc
         return msg
 
@@ -117,27 +122,26 @@ class OptionSet(dict):
     def __init__(self, *args):
         dict.__init__(self, *args)
         self.hm = self.buildHelpMessage()
-        
+
     def buildGetopt(self):
-        return ''.join([opt.buildGetopt() for opt in self.values()])
+        return ''.join([opt.buildGetopt() for opt in list(self.values())])
 
     def buildLongGetopt(self):
-        return [opt.buildLongGetopt() for opt in self.values() if opt.long]
+        return [opt.buildLongGetopt() for opt in list(self.values()) if opt.long]
 
     def buildHelpMessage(self):
         if hasattr(self, 'hm'):
             return self.hm
-        return ''.join([opt.buildHelpMessage() for opt in self.values()])
+        return '     '.join([opt.buildHelpMessage() for opt in list(self.values())])
 
     def helpExit(self, msg='', code=1):
         if msg:
-            print msg
-        print "Usage: %s" % self.buildHelpMessage()
+            print(msg)
+        print("Usage:\n     %s" % self.buildHelpMessage())
         raise SystemExit(code)
 
     def parse(self, argv, do_getopt=True):
         '''Parse options'''
-        ret = {}
         if do_getopt:
             try:
                 opts, args = getopt.getopt(argv, self.buildGetopt(),
@@ -147,7 +151,7 @@ class OptionSet(dict):
             if '-h' in argv:
                 self.helpExit('', 0)
             self['args'] = args
-        for key in self.keys():
+        for key in list(self.keys()):
             if key == 'args':
                 continue
             option = self[key]
@@ -173,18 +177,36 @@ DEBUG = Option("Enable debugging output", False, cmd='-d')
 VERBOSE = Option("Enable verbose output", False, cmd='-v')
 DAEMON = Option("Daemonize process, storing pid", False,
                 cmd='-D', odesc="<pidfile>")
+MDATA_OWNER = Option('Default ConfigFile owner',
+                     default='root', cf=('mdata', 'owner'),
+                     odesc='owner permissions')
+MDATA_GROUP = Option('Default ConfigFile group',
+                     default='root', cf=('mdata', 'group'),
+                     odesc='group permissions')
+MDATA_PERMS = Option('Default ConfigFile permissions',
+                     '644', cf=('mdata', 'perms'),
+                     odesc='octal permissions')
+MDATA_PARANOID = Option('Default ConfigFile paranoid setting',
+                     'false', cf=('mdata', 'paranoid'),
+                     odesc='ConfigFile paranoid setting')
 
 SERVER_REPOSITORY = Option('Server repository path', '/var/lib/bcfg2',
                            cf=('server', 'repository'), cmd='-Q',
-                           odesc='<repository path>' )
-SERVER_SVN = Option('Server svn support', False, cf=('server', 'svn'))
+                           odesc='<repository path>')
 SERVER_PLUGINS = Option('Server plugin list', cf=('server', 'plugins'),
-                           default=[], cook=list_split)
-SERVER_GENERATORS = Option('Server generator list', cf=('server', 'generators'),
-                           default=['SSHbase', 'Cfg', 'Pkgmgr', 'Rules'],
-                           cook=list_split)
-SERVER_STRUCTURES = Option('Server structure list', cf=('server', 'structures'),
-                           default=['Bundler', 'Base'], cook=list_split)
+                        # default server plugins
+                        default=[
+                                 'Base',
+                                 'Bundler',
+                                 'Cfg',
+                                 'Metadata',
+                                 'Pkgmgr',
+                                 'Rules',
+                                 'SSHbase',
+                                ],
+                        cook=list_split)
+SERVER_MCONNECT = Option('Server Metadata Connector list', cook=list_split,
+                         cf=('server', 'connectors'), default=['Probes'], )
 SERVER_FILEMONITOR = Option('Server file monitor', cf=('server', 'filemonitor'),
                             default='default', odesc='File monitoring driver')
 SERVER_LOCATION = Option('Server Location', cf=('components', 'bcfg2'),
@@ -193,15 +215,33 @@ SERVER_LOCATION = Option('Server Location', cf=('components', 'bcfg2'),
 SERVER_STATIC = Option('Server runs on static port', cf=('components', 'bcfg2'),
                        default=False, cook=bool_cook)
 SERVER_KEY = Option('Path to SSL key', cf=('communication', 'key'),
-                       default=False, cmd='-K', odesc='<ssl key file>')
+                    default=False, cmd='-K', odesc='<ssl key file>')
+SERVER_CERT = Option('Path to SSL certificate', default='/etc/bcfg2.key',
+                     cf=('communication', 'certificate'), odesc='<ssl cert>')
+SERVER_CA = Option('Path to SSL CA Cert', default=None,
+                   cf=('communication', 'ca'), odesc='<ca cert>')
+CLIENT_KEY = Option('Path to SSL key', cf=('communication', 'key'),
+                    default=None, cmd="--ssl-key", odesc='<ssl key>',
+                    long_arg=True)
+CLIENT_CERT = Option('Path to SSL certificate', default=None, cmd="--ssl-cert",
+                     cf=('communication', 'certificate'), odesc='<ssl cert>',
+                     long_arg=True)
+CLIENT_CA = Option('Path to SSL CA Cert', default=None, cmd="--ca-cert",
+                   cf=('communication', 'ca'), odesc='<ca cert>',
+                   long_arg=True)
+CLIENT_SCNS = Option('list of server commonNames', default=None, cmd="--ssl-cns",
+                     cf=('communication', 'serverCommonNames'),
+                     odesc='<commonName1:commonName2>', cook=list_split,
+                     long_arg=True)
+
 SERVER_PASSWORD = Option('Communication Password', cmd='-x', odesc='<password>',
                          cf=('communication', 'password'), default=False)
 INSTALL_PREFIX = Option('Installation location', cf=('server', 'prefix'),
-                       default=DEFAULT_INSTALL_PREFIX, odesc='</path>')
+                        default=DEFAULT_INSTALL_PREFIX, odesc='</path>')
 SERVER_PROTOCOL = Option('Server Protocol', cf=('communication', 'procotol'),
                          default='xmlrpc/ssl')
 SENDMAIL_PATH = Option('Path to sendmail', cf=('reports', 'sendmailpath'),
-                         default='/usr/lib/sendmail')
+                       default='/usr/lib/sendmail')
 
 CLIENT_PROFILE = Option('assert the given profile for the host',
                         default=False, cmd='-p', odesc="<profile>")
@@ -213,8 +253,9 @@ CLIENT_DRYRUN = Option('do not actually change the system',
 CLIENT_EXTRA_DISPLAY = Option('enable extra entry output',
                               default=False, cmd='-e', )
 CLIENT_PARANOID = Option('make automatic backups of config files',
-                         default=False, cmd='-P', )
-CLIENT_AGENT = Option('run in agent (continuous) mode, wait for reconfigure command from server', default=False, cmd='-A', )
+                         default=False, cmd='-P', cf=('client', 'paranoid'))
+CORE_PROFILE = Option('profile',
+                      default=False, cmd='-p', )
 CLIENT_DRIVERS = Option('Specify tool driver set', cmd='-D',
                         cf=('client', 'drivers'),
                         odesc="<driver1,driver2>", cook=list_split,
@@ -223,13 +264,14 @@ CLIENT_CACHE = Option('store the configuration in a file',
                       default=False, cmd='-c', odesc="<cache path>")
 CLIENT_REMOVE = Option('force removal of additional configuration items',
                        default=False, cmd='-r', odesc="<entry type|all>")
-CLIENT_BUNDLE = Option('only configure the given bundle', default=[],
-                       cmd='-b', odesc='<bundle>', cook=colon_split)
+CLIENT_BUNDLE = Option('only configure the given bundle(s)', default=[],
+                       cmd='-b', odesc='<bundle:bundle>', cook=colon_split)
+CLIENT_INDEP = Option('only configure the given bundle(s)', default=False,
+                       cmd='-z')
 CLIENT_KEVLAR = Option('run in kevlar (bulletproof) mode', default=False,
                        cmd='-k', )
 CLIENT_DLIST = Option('run client in server decision list mode', default=False,
-                       cmd='-l', odesc='<whitelist|blacklist>')
-CLIENT_BUILD = Option('run in build mode', default=False, cmd='-B', )
+                      cmd='-l', odesc='<whitelist|blacklist>')
 CLIENT_FILE = Option('configure from a file rather than querying the server',
                      default=False, cmd='-f', odesc='<specification path>')
 SERVER_FINGERPRINT = Option('Server Fingerprint', default=[], cmd='-F',
@@ -237,28 +279,35 @@ SERVER_FINGERPRINT = Option('Server Fingerprint', default=[], cmd='-F',
                             odesc='<f1,f2>', cook=flist_split)
 CLIENT_QUICK = Option('disable some checksum verification', default=False,
                       cmd='-q', )
-CLIENT_BACKGROUND = Option('Daemonize the agent', default=False, cmd='-i', )
-CLIENT_PORT = Option('the port on which to bind for agent mode', default='6789',
-                     cmd='-g', cf=('communication', 'agent-port'),
-                     odesc='<agent port>')
 CLIENT_USER = Option('the user to provide for authentication', default='root',
                      cmd='-u', cf=('communication', 'user'), odesc='<user>')
 INTERACTIVE = Option('prompt the user for each change', default=False,
                      cmd='-I', )
 
-AGENT_PORT = Option('Agent port', default=6789, cmd='-p', odesc='<port>',
-                    cf=('communication', 'agent-port'))
-AGENT_HOST = Option('Remote host', default=False, cmd='-H', odesc='<hostname>')
-
 ENCODING = Option('Encoding of cfg files', default=sys.getdefaultencoding(), cmd='-E', odesc='<encoding>',
                   cf=('components', 'encoding'))
 
+PARANOID_PATH = Option('Specify path for paranoid file backups',
+                       default='/var/cache/bcfg2', cf=('paranoid', 'path'),
+                       odesc='<paranoid backup path>')
+PARANOID_MAX_COPIES = Option('Specify the number of paranoid copies you want',
+                             default=1, cf=('paranoid', 'max_copies'),
+                             odesc='<max paranoid copies>')
+
 OMIT_LOCK_CHECK = Option('Omit lock check', default=False, cmd='-O')
 
-LOGGING_FILE_PATH = Option('Set path of file log', default=None, cmd='-o', odesc='<path>', cf=('logging', 'path'))
+LOGGING_FILE_PATH = Option('Set path of file log', default=None,
+                           cmd='-o', odesc='<path>', cf=('logging', 'path'))
+
+CLIENT_SERVICE_MODE = Option('Set client service mode', default='default',
+                             cmd='-s', odesc='<default|disabled|build>')
+
 
 class OptionParser(OptionSet):
-    '''OptionParser bootstraps option parsing, getting the value of the config file'''
+    '''
+       OptionParser bootstraps option parsing,
+       getting the value of the config file
+    '''
     def __init__(self, args):
         self.Bootstrap = OptionSet([('configfile', CFILE)])
         self.Bootstrap.parse(sys.argv[1:], do_getopt=False)
