@@ -1,19 +1,34 @@
 '''This module provides the baseclass for Bcfg2 Server Plugins'''
-__revision__ = '$Revision: 4932 $'
+__revision__ = '$Revision: 5651 $'
 
-import logging, lxml.etree, re, copy, posixpath
+import copy
+import logging
+import lxml.etree
+import os
+import posixpath
+import re
 
 from lxml.etree import XML, XMLSyntaxError
 
+import Bcfg2.Options
+
+# grab default metadata info from bcfg2.conf
+opts = {'owner': Bcfg2.Options.MDATA_OWNER,
+        'group': Bcfg2.Options.MDATA_GROUP,
+        'perms': Bcfg2.Options.MDATA_PERMS,
+        'paranoid': Bcfg2.Options.MDATA_PARANOID}
+mdata_setup = Bcfg2.Options.OptionParser(opts)
+mdata_setup.parse([])
+del mdata_setup['args']
+
 logger = logging.getLogger('Bcfg2.Plugin')
 
-default_file_metadata = {'owner': 'root', 'group': 'root', 'perms': '644',
-                         'encoding': 'ascii', 'paranoid':"false"}
+default_file_metadata = mdata_setup
 
 info_regex = re.compile( \
     '^owner:(\s)*(?P<owner>\S+)|group:(\s)*(?P<group>\S+)|' +
     'perms:(\s)*(?P<perms>\w+)|encoding:(\s)*(?P<encoding>\w+)|' +
-    '(?P<paranoid>paranoid(\s)*)|mtime:(\s)*(?P<mtime>\w+)$' )
+    'paranoid:(\s)*(?P<paranoid>\S+)|mtime:(\s)*(?P<mtime>\w+)$')
 
 class PluginInitError(Exception):
     '''Error raised in cases of Plugin initialization errors'''
@@ -24,9 +39,9 @@ class PluginExecutionError(Exception):
     pass
 
 class Plugin(object):
-    '''This is the base class for all Bcfg2 Server plugins. Several attributes must be defined
-    in the subclass:
-    __name__ : the name of the plugin
+    '''This is the base class for all Bcfg2 Server plugins.
+    Several attributes must be defined in the subclass:
+    name : the name of the plugin
     __version__ : a version string
     __author__ : the author/contact for the plugin
 
@@ -35,22 +50,50 @@ class Plugin(object):
       - Configuration entry binding (overloading HandlesEntry, or loads the Entries table)
       - Data collection (overloading GetProbes/ReceiveData)
     '''
-    __name__ = 'Plugin'
-    __version__ = '$Id: Plugin.py 4932 2008-10-08 17:56:34Z desai $'
+    name = 'Plugin'
+    __version__ = '$Id: Plugin.py 5651 2010-01-04 00:59:45Z solj $'
     __author__ = 'bcfg-dev@mcs.anl.gov'
-    __rmi__ = []
+    __rmi__ = ['toggle_debug']
     experimental = False
+    deprecated = False
 
     def __init__(self, core, datastore):
         object.__init__(self)
         self.Entries = {}
         self.core = core
-        self.data = "%s/%s" % (datastore, self.__name__)
-        self.logger = logging.getLogger('Bcfg2.Plugins.%s' % (self.__name__))
+        self.data = "%s/%s" % (datastore, self.name)
+        self.logger = logging.getLogger('Bcfg2.Plugins.%s' % (self.name))
+        self.running = True
+        self.debug_flag = False
 
-class GeneratorPlugin(Plugin):
+    def toggle_debug(self):
+        self.debug_flag = not self.debug_flag
+
+    def debug_log(self, message, flag=None):
+        if (flag is None) and self.debug_flag or flag:
+            self.logger.error(message)
+
+    @staticmethod
+    def make_path(path):
+        p_comp = path.split('/')
+        for i in range(2, len(p_comp) + 1):
+            ppath = '/' + '/'.join(p_comp[1:i])
+            try:
+                os.stat(ppath)
+            except:
+                os.mkdir(ppath)
+
+    @classmethod
+    def init_repo(cls, repo):
+        path = "%s/%s" % (repo, cls.name)
+        cls.make_path(path)
+
+    def shutdown(self):
+        self.running = False
+
+class Generator(object):
     '''Generator plugins contribute to literal client configurations'''
-    def HandlesEntry(self, entry):
+    def HandlesEntry(self, entry, metadata):
         '''This is the slow path method for routing configuration binding requests'''
         return False
 
@@ -58,26 +101,13 @@ class GeneratorPlugin(Plugin):
         '''This is the slow-path handler for configuration entry binding'''
         raise PluginExecutionError
 
-    def AcceptChoices(self, entry, metadata):
-        raise PluginExecutionError
-    
-    def AcceptPullData(self, specific, new_entry, verbose):
-        '''This is the null per-plugin implementation
-        of bcfg2-admin pull'''
-        raise PluginExecutionError
-
-    def CommitChanges(self):
-        '''Handle revctl commits, if needed'''
-        # not implemented yet
-        pass
-
-class StructurePlugin(Plugin):
+class Structure(object):
     '''Structure Plugins contribute to abstract client configurations'''
     def BuildStructures(self, metadata):
         '''return a list of abstract goal structures for client'''
         raise PluginExecutionError
 
-class MetadataPlugin(Plugin):
+class Metadata(object):
     '''Signal metadata capabilities for this plugin'''
     def add_client(self, client_name, attribs):
         '''add client'''
@@ -89,7 +119,23 @@ class MetadataPlugin(Plugin):
         '''create viz str for viz admin mode'''
         pass
 
-class ProbingPlugin(Plugin):
+    def get_initial_metadata(self, client_name):
+        raise PluginExecutionError
+
+    def merge_additional_data(self, imd, source, groups, data):
+        raise PluginExecutionError
+
+class Connector(object):
+    '''Connector Plugins augment client metadata instances'''
+    def get_additional_groups(self, metadata):
+        '''determine additional groups for metadata'''
+        return list()
+
+    def get_additional_data(self, metadata):
+        '''determine additional data for metadata inst'''
+        return dict()
+
+class Probing(object):
     '''Signal probe capability for this plugin'''
     def GetProbes(self, _):
         '''Return a set of probes for execution on client'''
@@ -99,25 +145,52 @@ class ProbingPlugin(Plugin):
         '''Receive probe results pertaining to client'''
         pass
 
-class StatisticsPlugin(Plugin):
+class Statistics(object):
     '''Signal statistics handling capability'''
-    def StoreStatistics(self, client, xdata):
-        pass
-
-    def WriteBack(self):
+    def process_statistics(self, client, xdata):
         pass
 
+class PullSource(object):
     def GetExtra(self, client):
         return []
 
     def GetCurrentEntry(self, client, e_type, e_name):
         raise PluginExecutionError
 
-class DecisionPlugin(Plugin):
+class PullTarget(object):
+    def AcceptChoices(self, entry, metadata):
+        raise PluginExecutionError
+
+    def AcceptPullData(self, specific, new_entry, verbose):
+        '''This is the null per-plugin implementation
+        of bcfg2-admin pull'''
+        raise PluginExecutionError
+
+class Decision(object):
     '''Signal decision handling capability'''
     def GetDecisions(self, metadata, mode):
         return []
 
+class ValidationError(Exception):
+    pass
+
+class StructureValidator(object):
+    '''Validate/modify goal structures'''
+    def validate_structures(self, metadata, structures):
+        raise ValidationError, "not implemented"
+
+class GoalValidator(object):
+    '''Validate/modify configuration goals'''
+    def validate_goals(self, metadata, goals):
+        raise ValidationError, "not implemented"
+
+class Version(object):
+    '''Interact with various version control systems'''
+    def get_revision(self):
+        return []
+    def commit_data(self):
+        pass
+
 # the rest of the file contains classes for coherent file caching
 
 class FileBacked(object):
@@ -125,20 +198,22 @@ class FileBacked(object):
     HandleEvent is called whenever fam registers an event.
     Index can parse the data into member data as required.
     This object is meant to be used as a part of DirectoryBacked.'''
-    
+
     def __init__(self, name):
         object.__init__(self)
         self.data = ''
         self.name = name
 
-    def HandleEvent(self, _=None):
+    def HandleEvent(self, event=None):
         '''Read file upon update'''
+        if event and event.code2str() not in ['exists', 'changed', 'created']:
+            return
         try:
             self.data = file(self.name).read()
             self.Index()
         except IOError:
             logger.error("Failed to read file %s" % (self.name))
-            
+
     def Index(self):
         '''Update local data structures based on current file state'''
         pass
@@ -166,10 +241,13 @@ class DirectoryBacked(object):
         '''Add new entry to data structures upon file creation'''
         if name == '':
             logger.info("got add for empty name")
-        elif self.entries.has_key(name):
+        elif name in self.entries:
             self.entries[name].HandleEvent()
         else:
-            if ((name[-1] == '~') or (name[:2] == '.#') or (name[-4:] == '.swp') or (name in ['SCCS', '.svn'])):
+            if ((name[-1] == '~') or
+                (name[:2] == '.#') or
+                (name[-4:] == '.swp') or
+                (name in ['SCCS', '.svn'])):
                 return
             if not self.patterns.match(name):
                 return
@@ -188,15 +266,17 @@ class DirectoryBacked(object):
         elif action == 'created':
             self.AddEntry(event.filename)
         elif action == 'changed':
-            if self.entries.has_key(event.filename):
+            if event.filename in self.entries:
                 self.entries[event.filename].HandleEvent(event)
         elif action == 'deleted':
-            if self.entries.has_key(event.filename):
+            if event.filename in self.entries:
                 del self.entries[event.filename]
         elif action in ['endExist']:
             pass
         else:
-            print "Got unknown event %s %s %s" % (event.requestID, event.code2str(), event.filename)
+            print "Got unknown event %s %s %s" % (event.requestID,
+                                                  event.code2str(),
+                                                  event.filename)
 
 class XMLFileBacked(FileBacked):
     '''This object is a coherent cache for an XML file to be used as a part of DirectoryBacked.'''
@@ -247,16 +327,16 @@ class StructFile(XMLFileBacked):
                                          and not isinstance(item, lxml.etree._Comment)]
             for group in [item for item in worklist if item.tag == 'Group']:
                 # if only python had forceable early-binding
-                if group.get('negate', 'false') == 'true':
+                if group.get('negate', 'false') in ['true', 'True']:
                     cmd = "lambda x:'%s' not in x.groups and predicate(x)"
                 else:
                     cmd = "lambda x:'%s' in x.groups and predicate(x)"
-                    
+
                 newpred = eval(cmd % (group.get('name')), {'predicate':predicate})
                 work[newpred] = group.getchildren()
 
     def Match(self, metadata):
-        '''Return matching fragments of independant'''
+        '''Return matching fragments of independent'''
         matching = [frag for (pred, frag) in self.fragments.iteritems() if pred(metadata)]
         if matching:
             return reduce(lambda x, y:x+y, matching)
@@ -271,7 +351,7 @@ class INode:
             'Group':"lambda x:'%s' not in x.groups and predicate(x)"}
     containers = ['Group', 'Client']
     ignore = []
-    
+
     def __init__(self, data, idict, parent=None):
         self.data = data
         self.contents = {}
@@ -279,7 +359,7 @@ class INode:
             self.predicate = lambda x:True
         else:
             predicate = parent.predicate
-            if data.get('negate', 'false') == 'true':
+            if data.get('negate', 'false') in ['true', 'True']:
                 psrc = self.nraw
             else:
                 psrc = self.raw
@@ -367,19 +447,20 @@ class XMLSrc(XMLFileBacked):
 
 class XMLDirectoryBacked(DirectoryBacked):
     '''Directorybacked for *.xml'''
-    patterns = re.compile('.*\.xml')    
+    patterns = re.compile('.*\.xml')
 
-class PrioDir(GeneratorPlugin, XMLDirectoryBacked):
+class PrioDir(Plugin, Generator, XMLDirectoryBacked):
     '''This is a generator that handles package assignments'''
-    __name__ = 'PrioDir'
+    name = 'PrioDir'
     __child__ = XMLSrc
 
     def __init__(self, core, datastore):
         Plugin.__init__(self, core, datastore)
+        Generator.__init__(self)
         try:
             XMLDirectoryBacked.__init__(self, self.data, self.core.fam)
         except OSError:
-            self.logger.error("Failed to load %s indices" % (self.__name__))
+            self.logger.error("Failed to load %s indices" % (self.name))
             raise PluginInitError
 
     def HandleEvent(self, event):
@@ -402,7 +483,7 @@ class PrioDir(GeneratorPlugin, XMLDirectoryBacked):
             self.logger.error("Called before data loaded")
             raise PluginExecutionError
         matching = [src for src in self.entries.values()
-                    if src.cache and src.cache[1].has_key(entry.tag)
+                    if src.cache and entry.tag in src.cache[1]
                     and src.cache[1][entry.tag].has_key(name)]
         if len(matching) == 0:
             raise PluginExecutionError
@@ -411,17 +492,19 @@ class PrioDir(GeneratorPlugin, XMLDirectoryBacked):
         else:
             prio = [int(src.priority) for src in matching]
             if prio.count(max(prio)) > 1:
-                self.logger.error("Found conflicting %s sources with same priority for %s, pkg %s" %
-                                  (entry.tag.lower(), metadata.hostname, entry.get('name')))
+                self.logger.error("Found conflicting sources with "
+                                  "same priority for %s, %s %s" %
+                                  (metadata.hostname,
+                                   entry.tag.lower(), entry.get('name')))
                 self.logger.error([item.name for item in matching])
-                self.logger.error("Prio was %s" % max(prio))
+                self.logger.error("Priority was %s" % max(prio))
                 raise PluginExecutionError
             index = prio.index(max(prio))
 
         data = matching[index].cache[1][entry.tag][name]
-        if data.has_key('__text__'):
+        if '__text__' in data:
             entry.text = data['__text__']
-        if data.has_key('__children__'):
+        if '__children__' in data:
             [entry.append(copy.deepcopy(item)) for item in data['__children__']]
         [entry.attrib.__setitem__(key, data[key]) for key in data.keys() \
          if not key.startswith('__')]
@@ -471,7 +554,7 @@ class Specificity:
         return False
 
 class SpecificData(object):
-    def __init__(self, name, _, specific, encoding):
+    def __init__(self, name, specific, encoding):
         self.name = name
         self.specific = specific
 
@@ -485,12 +568,11 @@ class SpecificData(object):
 
 class EntrySet:
     '''Entry sets deal with the host- and group-specific entries'''
-    ignore = re.compile("^(\.#.*|.*~|\\..*\\.(tmp|sw[px]))$")
-    def __init__(self, basename, path, props, entry_type, encoding):
+    ignore = re.compile("^(\.#.*|.*~|\\..*\\.(sw[px]))$")
+    def __init__(self, basename, path, entry_type, encoding):
         self.path = path
         self.entry_type = entry_type
         self.entries = {}
-        self.properties = props
         self.metadata = default_file_metadata.copy()
         self.infoxml = None
         self.encoding = encoding
@@ -536,7 +618,6 @@ class EntrySet:
                     logger.error("Could not process filename %s; ignoring" % fpath)
                 return
             self.entries[event.filename] = self.entry_type(fpath,
-                                                           self.properties,
                                                            spec, self.encoding)
         self.entries[event.filename].handle_event(event)
 
@@ -624,51 +705,26 @@ class EntrySet:
 
         raise PluginExecutionError
 
-# GroupSpool plugin common code (for TGenshi, TCheetah, and Cfg)
-
-class TemplateProperties(SingleXMLFileBacked):
-    '''Class for Genshi properties'''
-    def Index(self):
-        '''Build data into an elementtree object for templating usage'''
-        try:
-            self.properties = lxml.etree.XML(self.data)
-            del self.data
-        except lxml.etree.XMLSyntaxError:
-            logger.error("Failed to parse properties.xml; disabling")
-
-class FakeProperties:
-    '''Dummy class used when properties dont exist'''
-    def __init__(self):
-        self.properties = lxml.etree.Element("Properties")
-
-class GroupSpool(GeneratorPlugin):
+class GroupSpool(Plugin, Generator):
     '''The TGenshi generator implements a templating mechanism for configuration files'''
-    __name__ = 'GroupSpool'
-    __version__ = '$Id: Plugin.py 4932 2008-10-08 17:56:34Z desai $'
+    name = 'GroupSpool'
+    __version__ = '$Id: Plugin.py 5651 2010-01-04 00:59:45Z solj $'
     __author__ = 'bcfg-dev@mcs.anl.gov'
-    use_props = False
     filename_pattern = ""
     es_child_cls = object
     es_cls = EntrySet
 
     def __init__(self, core, datastore):
         Plugin.__init__(self, core, datastore)
+        Generator.__init__(self)
         if self.data[-1] == '/':
             self.data = self.data[:-1]
         self.Entries['ConfigFile'] = {}
+        self.Entries['Path'] = {}
         self.entries = {}
         self.handles = {}
         self.AddDirectoryMonitor('')
         self.encoding = core.encoding
-        if self.use_props:
-            try:
-                self.properties = TemplateProperties( \
-                    '%s/../etc/properties.xml' % (self.data), self.core.fam)
-            except:
-                self.properties = FakeProperties()
-                self.logger.info("%s properties disabled" % self.__name__)
-        else:
-            self.properties = FakeProperties()
 
     def HandleEvent(self, event):
         '''Unified FAM event handler for DirShadow'''
@@ -685,14 +741,14 @@ class GroupSpool(GeneratorPlugin):
         if action in ['exists', 'created']:
             if posixpath.isdir(epath):
                 self.AddDirectoryMonitor(epath[len(self.data):])
-            if ident not in self.entries:
-                dirpath  = "".join([self.data, ident])
+            if ident not in self.entries and posixpath.isfile(epath):
+                dirpath = "".join([self.data, ident])
                 self.entries[ident] = self.es_cls(self.filename_pattern,
                                                   dirpath,
-                                                  self.properties,
                                                   self.es_child_cls,
                                                   self.encoding)
-                self.Entries['ConfigFile'][ident] =  self.entries[ident].bind_entry
+                self.Entries['ConfigFile'][ident] = self.entries[ident].bind_entry
+                self.Entries['Path'][ident] = self.entries[ident].bind_entry
             if not posixpath.isdir(epath):
                 # do not pass through directory events
                 self.entries[ident].handle_event(event)
@@ -704,9 +760,10 @@ class GroupSpool(GeneratorPlugin):
                 # a directory was deleted
                 del self.entries[fbase]
                 del self.Entries['ConfigFile'][fbase]
+                del self.Entries['Path'][fbase]
             else:
                 self.entries[ident].handle_event(event)
-                                 
+
     def AddDirectoryMonitor(self, relative):
         '''Add new directory to FAM structures'''
         if not relative.endswith('/'):
