@@ -1,9 +1,10 @@
 '''This handles authentication setup'''
-__revision__ = '$Revision: 4680 $'
+__revision__ = '$Revision: 5075 $'
 
 import Bcfg2.Server.Plugin
 
-class Account(Bcfg2.Server.Plugin.GeneratorPlugin):
+class Account(Bcfg2.Server.Plugin.Plugin,
+              Bcfg2.Server.Plugin.Generator):
     '''This module generates account config files,
     based on an internal data repo:
     static.(passwd|group|limits.conf) -> static entries
@@ -12,21 +13,23 @@ class Account(Bcfg2.Server.Plugin.GeneratorPlugin):
     superusers -> users to be granted root privs on all hosts
     rootlike -> users to be granted root privs on some hosts
     '''
-    __name__ = 'Account'
-    __version__ = '$Id: Account.py 4680 2008-06-09 15:56:41Z desai $'
+    name = 'Account'
+    __version__ = '$Id: Account.py 5075 2009-02-11 21:16:53Z solj $'
     __author__ = 'bcfg-dev@mcs.anl.gov'
 
     def __init__(self, core, datastore):
         Bcfg2.Server.Plugin.Plugin.__init__(self, core, datastore)
+        Bcfg2.Server.Plugin.Generator.__init__(self)
         self.Entries = {'ConfigFile':{'/etc/passwd':self.from_yp_cb,
-                                           '/etc/group':self.from_yp_cb,
-                                           '/etc/security/limits.conf':self.gen_limits_cb,
-                                           '/root/.ssh/authorized_keys':self.gen_root_keys_cb,
-					   '/etc/sudoers':self.gen_sudoers}}
+                                      '/etc/group':self.from_yp_cb,
+                                      '/etc/security/limits.conf':self.gen_limits_cb,
+                                      '/root/.ssh/authorized_keys':self.gen_root_keys_cb,
+                                      '/etc/sudoers':self.gen_sudoers}}
         try:
             self.repository = Bcfg2.Server.Plugin.DirectoryBacked(self.data, self.core.fam)
         except:
-            self.logger.error("Failed to load repos: %s, %s" % (self.data, "%s/ssh" % (self.data)))
+            self.logger.error("Failed to load repos: %s, %s" % \
+                              (self.data, "%s/ssh" % (self.data)))
             raise Bcfg2.Server.Plugin.PluginInitError
 
     def from_yp_cb(self, entry, metadata):
@@ -35,16 +38,20 @@ class Account(Bcfg2.Server.Plugin.GeneratorPlugin):
         entry.text = self.repository.entries["static.%s" % (fname)].data
         entry.text += self.repository.entries["dyn.%s" % (fname)].data
         perms = {'owner':'root', 'group':'root', 'perms':'0644'}
-        [entry.attrib.__setitem__(key, value) for (key, value) in perms.iteritems()]
+        [entry.attrib.__setitem__(key, value) for (key, value) in \
+         perms.iteritems()]
 
     def gen_limits_cb(self, entry, metadata):
         '''Build limits entries based on current ACLs'''
         entry.text = self.repository.entries["static.limits.conf"].data
         superusers = self.repository.entries["superusers"].data.split()
-        useraccess = [line.split(':') for line in self.repository.entries["useraccess"].data.split()]
-        users = [user for (user, host) in useraccess if host == metadata.hostname.split('.')[0]]
+        useraccess = [line.split(':') for line in \
+                      self.repository.entries["useraccess"].data.split()]
+        users = [user for (user, host) in \
+                 useraccess if host == metadata.hostname.split('.')[0]]
         perms = {'owner':'root', 'group':'root', 'perms':'0600'}
-        [entry.attrib.__setitem__(key, value) for (key, value) in perms.iteritems()]
+        [entry.attrib.__setitem__(key, value) for (key, value) in \
+         perms.iteritems()]
         entry.text += "".join(["%s hard maxlogins 1024\n" % uname for uname in superusers + users])
         if "*" not in users:
             entry.text += "* hard maxlogins 0\n"
@@ -62,7 +69,7 @@ class Account(Bcfg2.Server.Plugin.GeneratorPlugin):
         rdata = self.repository.entries
         entry.text = "".join([rdata["%s.key" % user].data for user \
                               in superusers if \
-                              rdata.has_key("%s.key" % user)])
+                              ("%s.key" % user) in rdata])
         perms = {'owner':'root', 'group':'root', 'perms':'0600'}
         [entry.attrib.__setitem__(key, value) for (key, value) \
          in perms.iteritems()]
