@@ -1,7 +1,13 @@
 '''This module implements a config file repository'''
-__revision__ = '$Revision: 4959 $'
+__revision__ = '$Revision: 5510 $'
 
-import binascii, logging, os, re, tempfile, Bcfg2.Server.Plugin
+import binascii
+import logging
+import os
+import re
+import tempfile
+
+import Bcfg2.Server.Plugin
 
 logger = logging.getLogger('Bcfg2.Plugins.Cfg')
 
@@ -19,13 +25,17 @@ def process_delta(data, delta):
                 if line[1:] in datalines:
                     datalines.remove(line[1:])
         return "\n".join(datalines)
-    elif delta.op == 'diff':
-        basefile = open(tempfile.mktemp(), 'w')
+    elif delta.specific.delta == 'diff':
+        basehandle, basename = tempfile.mkstemp()
+        basefile = open(basename, 'w')
         basefile.write(data)
         basefile.close()
-        dfile = open(tempfile.mktemp(), 'w')
+        os.close(basehandle)
+        dhandle, dname = tempfile.mkstemp()
+        dfile = open(dname, 'w')
         dfile.write(delta.data)
         dfile.close()
+        os.close(dhandle)
         ret = os.system("patch -uf %s < %s > /dev/null 2>&1" \
                         % (basefile.name, dfile.name))
         output = open(basefile.name, 'r').read()
@@ -38,7 +48,7 @@ class CfgMatcher:
     def __init__(self, fname):
         name = re.escape(fname)
         self.basefile_reg = re.compile('^(?P<basename>%s)(|\\.H_(?P<hostname>\S+)|.G(?P<prio>\d+)_(?P<group>\S+))$' % name)
-        self.delta_reg = re.compile('^(?P<basename>%s)(|\\.H_(?P<hostname>\S+)|\\.G(?P<prio>\d+)_(?P<group>\S+))\\.(?P<delta>(cat|diff))$' % fname)
+        self.delta_reg = re.compile('^(?P<basename>%s)(|\\.H_(?P<hostname>\S+)|\\.G(?P<prio>\d+)_(?P<group>\S+))\\.(?P<delta>(cat|diff))$' % name)
         self.cat_count = fname.count(".cat")
         self.diff_count = fname.count(".diff")
 
@@ -49,8 +59,9 @@ class CfgMatcher:
         return self.basefile_reg.match(fname)
 
 class CfgEntrySet(Bcfg2.Server.Plugin.EntrySet):
-    def __init__(self, basename, path, props, entry_type, encoding):
-        Bcfg2.Server.Plugin.EntrySet.__init__(self, basename, path, props, entry_type, encoding)
+    def __init__(self, basename, path, entry_type, encoding):
+        Bcfg2.Server.Plugin.EntrySet.__init__(self, basename, path,
+                                              entry_type, encoding)
         self.specific = CfgMatcher(path.split('/')[-1])
 
     def sort_by_specific(self, one, other):
@@ -74,9 +85,13 @@ class CfgEntrySet(Bcfg2.Server.Plugin.EntrySet):
         used = self.get_pertinent_entries(metadata)
         basefile = used.pop(0)
         data = basefile.data
+        if entry.tag == 'Path':
+            entry.set('type', 'file')
         for delta in used:
             data = data.strip()
             data = process_delta(data, delta)
+        if used:
+            data += '\n'
         if entry.get('encoding') == 'base64':
             entry.text = binascii.b2a_base64(data)
         else:
@@ -125,12 +140,12 @@ class CfgEntrySet(Bcfg2.Server.Plugin.EntrySet):
             if log:
                 logger.info("Wrote file %s" % infofile.name)
 
-class Cfg(Bcfg2.Server.Plugin.GroupSpool):
+class Cfg(Bcfg2.Server.Plugin.GroupSpool,
+          Bcfg2.Server.Plugin.PullTarget):
     '''This generator in the configuration file repository for bcfg2'''
-    __name__ = 'Cfg'
-    __version__ = '$Id: Cfg.py 4959 2008-10-31 19:20:45Z desai $'
+    name = 'Cfg'
+    __version__ = '$Id: Cfg.py 5510 2009-10-29 01:27:51Z desai $'
     __author__ = 'bcfg-dev@mcs.anl.gov'
-    use_props = False
     es_cls = CfgEntrySet
     es_child_cls = Bcfg2.Server.Plugin.SpecificData
 
@@ -139,4 +154,3 @@ class Cfg(Bcfg2.Server.Plugin.GroupSpool):
 
     def AcceptPullData(self, specific, new_entry, log):
         return self.entries[new_entry.get('name')].write_update(specific, new_entry, log)
-
