@@ -1,13 +1,15 @@
 '''This plugin provides automatic dependency handling'''
-__revision__ = '$Revision:$'
+__revision__ = '$Revision: 5477 $'
 
-import Bcfg2.Server.Plugin, lxml.etree
+import lxml.etree
+
+import Bcfg2.Server.Plugin
 
 class DNode(Bcfg2.Server.Plugin.INode):
     '''DNode provides supports for single predicate types for dependencies'''
     raw = {'Group':"lambda x:'%s' in x.groups and predicate(x)"}
     containers = ['Group']
-    
+
     def __init__(self, data, idict, parent=None):
         self.data = data
         self.contents = {}
@@ -34,21 +36,23 @@ class DNode(Bcfg2.Server.Plugin.INode):
 class DepXMLSrc(Bcfg2.Server.Plugin.XMLSrc):
     __node__ = DNode
 
-class Deps(Bcfg2.Server.Plugin.PrioDir):
-    __name__ = 'Deps'
-    __version__ = '$Id:$'
+class Deps(Bcfg2.Server.Plugin.PrioDir,
+           Bcfg2.Server.Plugin.StructureValidator):
+    name = 'Deps'
+    __version__ = '$Id: Deps.py 5477 2009-10-07 14:46:08Z solj $'
     __author__ = 'bcfg-dev@mcs.anl.gov'
     __child__ = DepXMLSrc
 
     def __init__(self, core, datastore):
         Bcfg2.Server.Plugin.PrioDir.__init__(self, core, datastore)
+        Bcfg2.Server.Plugin.StructureValidator.__init__(self)
         self.cache = {}
 
     def HandleEvent(self, event):
         self.cache = {}
         Bcfg2.Server.Plugin.PrioDir.HandleEvent(self, event)
-        
-    def GeneratePrereqs(self, structures, metadata):
+
+    def validate_structures(self, metadata, structures):
         entries = []
         prereqs = []
         for structure in structures:
@@ -58,20 +62,20 @@ class Deps(Bcfg2.Server.Plugin.PrioDir):
                     entries.append((entry.tag, entry.get('name')))
         entries.sort()
         entries = tuple(entries)
-        gdata = metadata.groups[:]
+        gdata = list(metadata.groups)
         gdata.sort()
         gdata = tuple(gdata)
-        if self.cache.has_key((entries, gdata)):
+        if (entries, gdata) in self.cache:
             prereqs = self.cache[(entries, gdata)]
         else:
             [src.Cache(metadata) for src in self.entries.values()]
-            
+
             toexamine = list(entries[:])
             while toexamine:
                 entry = toexamine.pop()
                 matching = [src for src in self.entries.values()
-                            if src.cache and src.cache[1].has_key(entry[0])
-                            and src.cache[1][entry[0]].has_key(entry[1])]
+                            if src.cache and entry[0] in src.cache[1]
+                            and entry[1] in src.cache[1][entry[0]]]
                 if len(matching) > 1:
                     prio = [int(src.priority) for src in matching]
                     if prio.count(max(prio)) > 1:
@@ -89,11 +93,11 @@ class Deps(Bcfg2.Server.Plugin.PrioDir):
                             toexamine.append(prq)
                             prereqs.append(prq)
             self.cache[(entries, gdata)] = prereqs
-            
+
         newstruct = lxml.etree.Element("Independent")
         for tag, name in prereqs:
             try:
                 lxml.etree.SubElement(newstruct, tag, name=name)
             except:
                 self.logger.error("Failed to add dep entry for %s:%s" % (tag, name))
-        return newstruct
+        structures.append(newstruct)
