@@ -1,7 +1,12 @@
 '''This file stores persistent metadata for the Bcfg2 Configuration Repository'''
-__revision__ = '$Revision: 4942 $'
+__revision__ = '$Revision: 5673 $'
 
-import lxml.etree, re, socket, time, fcntl, copy
+import copy
+import fcntl
+import lxml.etree
+import os
+import socket
+import time
 import Bcfg2.Server.Plugin
 
 class MetadataConsistencyError(Exception):
@@ -12,100 +17,56 @@ class MetadataRuntimeError(Exception):
     '''This error is raised when the metadata engine is called prior to reading enough data'''
     pass
 
-probe_matcher = re.compile("(?P<basename>\S+)(.(?P<mode>[GH])_\S+)?")
-
 class ClientMetadata(object):
     '''This object contains client metadata'''
-    def __init__(self, client, groups, bundles, categories, probed, uuid,
-                 password, overall):
+    def __init__(self, client, profile, groups, bundles,
+                 aliases, addresses, categories, uuid, password, query):
         self.hostname = client
+        self.profile = profile
         self.bundles = bundles
+        self.aliases = aliases
+        self.addresses = addresses
         self.groups = groups
         self.categories = categories
-        self.probes = probed
         self.uuid = uuid
         self.password = password
-        self.all = overall
+        self.connectors = []
+        self.query = query
 
     def inGroup(self, group):
         '''Test to see if client is a member of group'''
         return group in self.groups
 
-    def get_clients_by_group(self, group):
-        """
-        return a list of clients that are members of a group
-        Arguments:
-        - `group`: group name
-        """
-        profiles = [key for key, value in self.all[0].iteritems() \
-                    if group in value[1]]
-        return [key for key, value in self.all[1].iteritems() \
-                if value in profiles]
+class MetadataQuery(object):
+    def __init__(self, by_name, get_clients, by_groups, by_profiles, all_groups):
+        # resolver is set later
+        self.by_name = by_name
+        self.names_by_groups = by_groups
+        self.names_by_profiles = by_profiles
+        self.all_clients = get_clients
+        self.all_groups = all_groups
 
-    def get_clients_by_profile(self, profile):
-        """
-        return clients with a given profile
-        Arguments:
-        - `profile`: profile name
-        """
-        return [key for key, value in self.all[1].iteritems() \
-                if value == profile]
+    def by_groups(self, groups):
+        return [self.by_name(name) for name in self.names_by_groups(groups)]
 
-class ProbeSet(Bcfg2.Server.Plugin.EntrySet):
-    def __init__(self, path, fam, encoding, plugin_name):
-        fpattern = '[0-9A-Za-z_\-]+'
-        self.plugin_name = plugin_name
-        Bcfg2.Server.Plugin.EntrySet.__init__(self, fpattern, path, True, 
-                                              Bcfg2.Server.Plugin.SpecificData,
-                                              encoding)
-        fam.AddMonitor(path, self)
-        self.bangline = re.compile('^#!(?P<interpreter>(/\w+)+)$')
+    def by_profiles(self, profiles):
+        return [self.by_name(name) for name in self.names_by_profiles(profiles)]
 
-    def HandleEvent(self, event):
-        if event.filename != self.path:
-            return self.handle_event(event)
+    def all(self):
+        return [self.by_name(name) for name in self.all_clients()]
 
-    def get_probe_data(self, metadata):
-        ret = []
-        candidates = self.get_matching(metadata)
-        temp = {}
-        for cand in candidates:
-            if cand.specific.all:
-                if cand.name not in temp:
-                    temp[cand.name] = (cand, 0)
-                continue
-            mdata = probe_matcher.match(cand.name).groupdict()
-            if mdata['basename'] in temp:
-                if mdata['mode'] > temp[mdata['basename']][1]:
-                    temp[mdata['basename']] = (cand, mdata['mode'])
-            else:
-                temp[mdata['basename']] = (cand, mdata['mode'])
-        
-        for (name, data) in temp.iteritems():
-            entry, prio = data
-            probe = lxml.etree.Element('probe')
-            probe.set('name', name.split('/')[-1])
-            probe.set('source', self.plugin_name)
-            probe.text = entry.data
-            match = self.bangline.match(entry.data.split('\n')[0])
-            if match:
-                probe.set('interpreter', match.group('interpreter'))
-            else:
-                probe.set('interpreter', '/bin/sh')
-            ret.append(probe)
-        return ret
-
-
-class Metadata(Bcfg2.Server.Plugin.MetadataPlugin,
-               Bcfg2.Server.Plugin.ProbingPlugin):
+class Metadata(Bcfg2.Server.Plugin.Plugin,
+               Bcfg2.Server.Plugin.Metadata,
+               Bcfg2.Server.Plugin.Statistics):
     '''This class contains data for bcfg2 server metadata'''
-    __version__ = '$Id: Metadata.py 4942 2008-10-10 07:20:42Z solj $'
+    __version__ = '$Id: Metadata.py 5673 2010-01-13 01:54:54Z solj $'
     __author__ = 'bcfg-dev@mcs.anl.gov'
-    __name__ = "Metadata"
+    name = "Metadata"
 
     def __init__(self, core, datastore, watch_clients=True):
         Bcfg2.Server.Plugin.Plugin.__init__(self, core, datastore)
-        #self.__name__ = 'Metadata'
+        Bcfg2.Server.Plugin.Metadata.__init__(self)
+        Bcfg2.Server.Plugin.Statistics.__init__(self)
         if watch_clients:
             try:
                 core.fam.AddMonitor("%s/%s" % (self.data, "groups.xml"), self)
@@ -116,11 +77,13 @@ class Metadata(Bcfg2.Server.Plugin.MetadataPlugin,
         if watch_clients:
             self.states = {"groups.xml":False, "clients.xml":False}
         self.addresses = {}
+        self.auth = dict()
         self.clients = {}
         self.aliases = {}
         self.groups = {}
         self.cgroups = {}
         self.public = []
+        self.private = []
         self.profiles = []
         self.categories = {}
         self.bad_clients = {}
@@ -133,15 +96,22 @@ class Metadata(Bcfg2.Server.Plugin.MetadataPlugin,
         self.clientdata_original = None
         self.default = None
         self.pdirty = False
-        try:
-            loc = datastore + "/Probes"
-            self.probes = ProbeSet(loc, core.fam, core.encoding, self.__name__)
-        except:
-            self.probes = False
-        self.probedata = {}
         self.extra = {'groups.xml':[], 'clients.xml':[]}
         self.password = core.password
-        self.load_probedata()
+        self.query = MetadataQuery(core.build_metadata,
+                                   lambda:self.clients.keys(),
+                                   self.get_client_names_by_groups,
+                                   self.get_client_names_by_profiles,
+                                   self.get_all_group_names)
+
+    @classmethod
+    def init_repo(cls, repo, groups, os_selection, clients):
+        path = '%s/%s' % (repo, cls.name)
+        cls.make_path(path)
+        open("%s/Metadata/groups.xml" %
+             repo, "w").write(groups % os_selection)
+        open("%s/Metadata/clients.xml" %
+             repo, "w").write(clients % socket.getfqdn())
 
     def get_groups(self):
         '''return groups xml tree'''
@@ -149,6 +119,130 @@ class Metadata(Bcfg2.Server.Plugin.MetadataPlugin,
         root = groups_tree.getroot()
         return root
 
+    def search_group(self, group_name, tree):
+        '''find a group'''
+        for node in tree.findall("//Group"):
+            if node.get("name") == group_name:
+                return node
+            for child in node:
+                if child.tag == "Alias" and child.attrib["name"] == group_name:
+                    return node
+        return None
+
+    def add_group(self, group_name, attribs):
+        '''add group to groups.xml'''
+        tree = lxml.etree.parse(self.data + "/groups.xml")
+        root = tree.getroot()
+        element = lxml.etree.Element("Group", name=group_name)
+        for key, val in attribs.iteritems():
+            element.set(key, val)
+        node = self.search_group(group_name, tree)
+        if node != None:
+            self.logger.error("Group \"%s\" already exists" % (group_name))
+            raise MetadataConsistencyError
+        root.append(element)
+        group_tree = open(self.data + "/groups.xml","w")
+        fd = group_tree.fileno()
+        while True:
+            try:
+                fcntl.lockf(fd, fcntl.LOCK_EX | fcntl.LOCK_NB)
+            except IOError:
+                continue
+            else:
+                break
+        tree.write(group_tree)
+        fcntl.lockf(fd, fcntl.LOCK_UN)
+        group_tree.close()
+
+    def update_group(self, group_name, attribs):
+        '''Update a groups attributes'''
+        tree = lxml.etree.parse(self.data + "/groups.xml")
+        root = tree.getroot()
+        node = self.search_group(group_name, tree)
+        if node == None:
+            self.logger.error("Group \"%s\" not found" % (group_name))
+            raise MetadataConsistencyError
+        node.attrib.update(attribs)
+        group_tree = open(self.data + "/groups.xml","w")
+        fd = group_tree.fileno()
+        while True:
+            try:
+                fcntl.lockf(fd, fcntl.LOCK_EX | fcntl.LOCK_NB)
+            except IOError:
+                continue
+            else:
+                break
+        tree.write(group_tree)
+        fcntl.lockf(fd, fcntl.LOCK_UN)
+        group_tree.close()
+
+    def remove_group(self, group_name):
+        '''Remove a group'''
+        tree = lxml.etree.parse(self.data + "/groups.xml")
+        root = tree.getroot()
+        node = self.search_group(group_name, tree)
+        if node == None:
+            self.logger.error("Client \"%s\" not found" % (group_name))
+            raise MetadataConsistencyError
+        root.remove(node)
+        group_tree = open(self.data + "/groups.xml","w")
+        fd = group_tree.fileno()
+        while True:
+            try:
+                fcntl.lockf(fd, fcntl.LOCK_EX | fcntl.LOCK_NB)
+            except IOError:
+                continue
+            else:
+                break
+        tree.write(group_tree)
+        fcntl.lockf(fd, fcntl.LOCK_UN)
+        group_tree.close()
+
+    def add_bundle(self, bundle_name):
+        '''add bundle to groups.xml'''
+        tree = lxml.etree.parse(self.data + "/groups.xml")
+        root = tree.getroot()
+        element = lxml.etree.Element("Bundle", name=bundle_name)
+        node = self.search_group(bundle_name, tree)
+        if node != None:
+            self.logger.error("Bundle \"%s\" already exists" % (bundle_name))
+            raise MetadataConsistencyError
+        root.append(element)
+        group_tree = open(self.data + "/groups.xml","w")
+        fd = group_tree.fileno()
+        while True:
+            try:
+                fcntl.lockf(fd, fcntl.LOCK_EX | fcntl.LOCK_NB)
+            except IOError:
+                continue
+            else:
+                break
+        tree.write(group_tree)
+        fcntl.lockf(fd, fcntl.LOCK_UN)
+        group_tree.close()
+
+    def remove_bundle(self, bundle_name):
+        '''Remove a bundle'''
+        tree = lxml.etree.parse(self.data + "/groups.xml")
+        root = tree.getroot()
+        node = self.search_group(bundle_name, tree)
+        if node == None:
+            self.logger.error("Bundle \"%s\" not found" % (bundle_name))
+            raise MetadataConsistencyError
+        root.remove(node)
+        group_tree = open(self.data + "/groups.xml","w")
+        fd = group_tree.fileno()
+        while True:
+            try:
+                fcntl.lockf(fd, fcntl.LOCK_EX | fcntl.LOCK_NB)
+            except IOError:
+                continue
+            else:
+                break
+        tree.write(group_tree)
+        fcntl.lockf(fd, fcntl.LOCK_UN)
+        group_tree.close()
+
     def search_client(self, client_name, tree):
         '''find a client'''
         for node in tree.findall("//Client"):
@@ -184,15 +278,15 @@ class Metadata(Bcfg2.Server.Plugin.MetadataPlugin,
         fcntl.lockf(fd, fcntl.LOCK_UN)
         client_tree.close()
 
-    def remove_client(self, client_name):
-        '''Remove a client'''
+    def update_client(self, client_name, attribs):
+        '''Update a clients attributes'''
         tree = lxml.etree.parse(self.data + "/clients.xml")
         root = tree.getroot()
         node = self.search_client(client_name, tree)
         if node == None:
             self.logger.error("Client \"%s\" not found" % (client_name))
             raise MetadataConsistencyError
-        root.remove(node)
+        node.attrib.update(attribs)
         client_tree = open(self.data + "/clients.xml","w")
         fd = client_tree.fileno()
         while True:
@@ -211,7 +305,7 @@ class Metadata(Bcfg2.Server.Plugin.MetadataPlugin,
         filename = event.filename.split('/')[-1]
         if filename in ['groups.xml', 'clients.xml']:
             dest = filename
-        elif filename in reduce(lambda x,y:x+y, self.extra.values()):
+        elif filename in reduce(lambda x, y:x+y, self.extra.values()):
             if event.code2str() == 'exists':
                 return
             dest = [key for key, value in self.extra.iteritems() if filename in value][0]
@@ -240,23 +334,31 @@ class Metadata(Bcfg2.Server.Plugin.MetadataPlugin,
         if dest == 'clients.xml':
             self.clients = {}
             self.aliases = {}
+            self.raliases = {}
             self.bad_clients = {}
             self.secure = []
             self.floating = []
             self.addresses = {}
+            self.raddresses = {}
             self.clientdata_original = xdata_original
             self.clientdata = xdata
             for client in xdata.findall('.//Client'):
                 clname = client.get('name').lower()
                 if 'address' in client.attrib:
                     caddr = client.get('address')
-                    if self.addresses.has_key(caddr):
+                    if caddr in self.addresses:
                         self.addresses[caddr].append(clname)
                     else:
                         self.addresses[caddr] = [clname]
+                    if clname not in self.raddresses:
+                        self.raddresses[clname] = set()
+                    self.raddresses[clname].add(caddr)
+                if 'auth' in client.attrib:
+                    self.auth[client.get('name')] = client.get('auth',
+                                                               'cert+password')
                 if 'uuid' in client.attrib:
                     self.uuid[client.get('uuid')] = clname
-                if client.get('secure', 'false') == 'true' :
+                if client.get('secure', 'false') == 'true':
                     self.secure.append(clname)
                 if client.get('location', 'fixed') == 'floating':
                     self.floating.append(clname)
@@ -264,16 +366,22 @@ class Metadata(Bcfg2.Server.Plugin.MetadataPlugin,
                     self.passwords[clname] = client.get('password')
                 for alias in [alias for alias in client.findall('Alias')\
                               if 'address' in alias.attrib]:
-                    if self.addresses.has_key(alias.get('address')):
+                    if alias.get('address') in self.addresses:
                         self.addresses[alias.get('address')].append(clname)
                     else:
                         self.addresses[alias.get('address')] = [clname]
-                    
+                    if clname not in self.raddresses:
+                        self.raddresses[clname] = set()
+                    self.raddresses[clname].add(alias.get('address'))
                 self.clients.update({clname: client.get('profile')})
                 [self.aliases.update({alias.get('name'): clname}) \
                  for alias in client.findall('Alias')]
+                self.raliases[clname] = set()
+                [self.raliases[clname].add(alias.get('name')) for alias \
+                 in client.findall('Alias')]
         elif dest == 'groups.xml':
             self.public = []
+            self.private = []
             self.profiles = []
             self.groups = {}
             grouptmp = {}
@@ -289,33 +397,33 @@ class Metadata(Bcfg2.Server.Plugin.MetadataPlugin,
                     self.profiles.append(group.get('name'))
                 if group.get('public', 'false') == 'true':
                     self.public.append(group.get('name'))
-                if group.attrib.has_key('category'):
+                elif group.get('public', 'true') == 'false':
+                    self.private.append(group.get('name'))
+                if 'category' in group.attrib:
                     self.categories[group.get('name')] = group.get('category')
             for group in grouptmp:
                 # self.groups[group] => (bundles, groups, categories)
-                self.groups[group] = ([], [], {})
+                self.groups[group] = (set(), set(), {})
                 tocheck = [group]
                 group_cat = self.groups[group][2]
                 while tocheck:
                     now = tocheck.pop()
-                    if now not in self.groups[group][1]:
-                        self.groups[group][1].append(now)
-                    if grouptmp.has_key(now):
+                    self.groups[group][1].add(now)
+                    if now in grouptmp:
                         (bundles, groups) = grouptmp[now]
                         for ggg in [ggg for ggg in groups if ggg not in self.groups[group][1]]:
                             if ggg not in self.categories or \
                                    self.categories[ggg] not in self.groups[group][2]:
-                                self.groups[group][1].append(ggg)
+                                self.groups[group][1].add(ggg)
                                 tocheck.append(ggg)
-                                if self.categories.has_key(ggg):
+                                if ggg in self.categories:
                                     group_cat[self.categories[ggg]] = ggg
                             elif ggg in self.categories:
                                 self.logger.info("Group %s: %s cat-suppressed %s" % \
                                                  (group,
                                                   group_cat[self.categories[ggg]],
                                                   ggg))
-                        [self.groups[group][0].append(bund) for bund in bundles
-                         if bund not in self.groups[group][0]]
+                        [self.groups[group][0].add(bund) for bund in bundles]
         self.states[dest] = True
         if False not in self.states.values():
             # check that all client groups are real and complete
@@ -341,17 +449,18 @@ class Metadata(Bcfg2.Server.Plugin.MetadataPlugin,
         if profile not in self.public:
             self.logger.error("Failed to set client %s to private group %s" % (client, profile))
             raise MetadataConsistencyError
-        if self.clients.has_key(client):
+        if client in self.clients:
             self.logger.info("Changing %s group from %s to %s" % (client, self.clients[client], profile))
             cli = self.clientdata_original.xpath('.//Client[@name="%s"]' % (client))
             cli[0].set('profile', profile)
         else:
             self.logger.info("Creating new client: %s, profile %s" % \
                              (client, profile))
-            if self.session_cache.has_key(addresspair):
+            if addresspair in self.session_cache:
                 # we are working with a uuid'd client
                 lxml.etree.SubElement(self.clientdata_original.getroot(),
-                                      'Client', name=client,
+                                      'Client',
+                                      name=self.session_cache[addresspair][1],
                                       uuid=client, profile=profile,
                                       address=addresspair[0])
             else:
@@ -364,72 +473,59 @@ class Metadata(Bcfg2.Server.Plugin.MetadataPlugin,
     def write_back_clients(self):
         '''Write changes to client.xml back to disk'''
         try:
-            datafile = open("%s/%s" % (self.data, 'clients.xml'), 'w')
+            datafile = open("%s/%s" % (self.data, 'clients.xml.new'), 'w')
         except IOError:
-            self.logger.error("Failed to write clients.xml")
+            self.logger.error("Failed to write clients.xml.new")
             raise MetadataRuntimeError
+        # prep data
+        dataroot = self.clientdata_original.getroot()
+        if hasattr(dataroot, 'iter'):
+            items = dataroot.iter()
+        else:
+            items = dataroot.getchildren()
+        for item in items:
+            # no items have text data of any sort
+            item.tail = None
+            item.text = None
+        newcontents = lxml.etree.tostring(dataroot, pretty_print=True)
+        if len(newcontents) == 0:
+            self.logger.error("Metadata: lxml returns 0 length string")
+            raise MetadataRuntimeError
+
         fd = datafile.fileno()
         while self.locked(fd) == True:
             pass
-        datafile.write(lxml.etree.tostring(self.clientdata_original.getroot(),
-                                           pretty_print='true'))
-        fcntl.lockf(fd, fcntl.LOCK_UN)
-        datafile.close()
-    
+        try:
+            datafile.write(newcontents)
+        except:
+            fcntl.lockf(fd, fcntl.LOCK_UN)
+            self.logger.error("Metadata: Failed to write new clients data to clients.xml.new", exc_info=1)
+            os.unlink("%s/%s" % (self.data, "clients.xml.new"))
+            raise MetadataRuntimeError
+        datafile.close()            
+
+        try:
+            os.rename("%s/%s" % (self.data, 'clients.xml.new'),
+                      "%s/%s" % (self.data, 'clients.xml'))
+        except:
+            self.logger.error("Metadata: Failed to rename clients.xml.new")
+            raise MetadataRuntimeError
+
     def locked(self, fd):
         try:
             fcntl.lockf(fd, fcntl.LOCK_EX | fcntl.LOCK_NB)
         except IOError:
             return True
         return False
-    
-    def write_probedata(self):
-        '''write probe data out for use with bcfg2-info'''
-        if self.pdirty:
-            top = lxml.etree.Element("Probed")
-            for client, probed in self.probedata.iteritems():
-                cx = lxml.etree.SubElement(top, 'Client', name=client)
-                for probe in probed:
-                    lxml.etree.SubElement(cx, 'Probe', name=probe,
-                                          value=self.probedata[client][probe])
-                for group in self.cgroups[client]:
-                    lxml.etree.SubElement(cx, "Group", name=group)
-            data = lxml.etree.tostring(top, encoding='UTF-8', xml_declaration=True,
-                                       pretty_print='true')
-            try:
-                datafile = open("%s/%s" % (self.data, 'probed.xml'), 'w')
-            except IOError:
-                self.logger.error("Failed to write probed.xml")
-                raise MetadataRuntimeError
-            datafile.write(data)
-            self.pdirty = False
-
-    def load_probedata(self):
-        try:
-            data = lxml.etree.parse(self.data + '/probed.xml').getroot()
-        except:
-            self.logger.error("Failed to read file probed.xml")
-            return
-        self.probedata = {}
-        self.cgroups = {}
-        for client in data.getchildren():
-            self.probedata[client.get('name')] = {}
-            self.cgroups[client.get('name')]=[]
-            for pdata in client:
-                if (pdata.tag == 'Probe'):
-                    self.probedata[client.get('name')][pdata.get('name')] = pdata.get('value')
-                elif (pdata.tag == 'Group'):
-                    self.cgroups[client.get('name')].append(pdata.get('name'))
 
     def resolve_client(self, addresspair):
         '''Lookup address locally or in DNS to get a hostname'''
-        #print self.session_cache
-        if self.session_cache.has_key(addresspair):
+        if addresspair in self.session_cache:
             (stamp, uuid) = self.session_cache[addresspair]
-            if time.time() - stamp < 60:
-                return self.uuid[uuid]
+            if time.time() - stamp < 90:
+                return self.session_cache[addresspair][1]
         address = addresspair[0]
-        if self.addresses.has_key(address):
+        if address in self.addresses:
             if len(self.addresses[address]) != 1:
                 self.logger.error("Address %s has multiple reverse assignments; a uuid must be used" % (address))
                 raise MetadataConsistencyError
@@ -443,26 +539,29 @@ class Metadata(Bcfg2.Server.Plugin.MetadataPlugin,
             warning = "address resolution error for %s" % (address)
             self.logger.warning(warning)
             raise MetadataConsistencyError
-    
-    def get_metadata(self, client):
+
+    def get_initial_metadata(self, client):
         '''Return the metadata for a given client'''
         client = client.lower()
-        if self.aliases.has_key(client):
+        if client in self.aliases:
             client = self.aliases[client]
-        if self.clients.has_key(client):
-            (bundles, groups, categories) = self.groups[self.clients[client]]
+        if client in self.clients:
+            profile = self.clients[client]
+            (bundles, groups, categories) = self.groups[profile]
         else:
             if self.default == None:
                 self.logger.error("Cannot set group for client %s; no default group set" % (client))
                 raise MetadataConsistencyError
             self.set_profile(client, self.default, (None, None))
+            profile = self.default
             [bundles, groups, categories] = self.groups[self.default]
-        probed = self.probedata.get(client, {})
-        newgroups = groups[:]
-        newbundles = bundles[:]
+        aliases = self.raliases.get(client, set())
+        addresses = self.raddresses.get(client, set())
+        newgroups = set(groups)
+        newbundles = set(bundles)
         newcategories = {}
         newcategories.update(categories)
-        if self.passwords.has_key(client):
+        if client in self.passwords:
             password = self.passwords[client]
         else:
             password = None
@@ -472,69 +571,95 @@ class Metadata(Bcfg2.Server.Plugin.MetadataPlugin,
         else:
             uuid = None
         for group in self.cgroups.get(client, []):
-            if self.groups.has_key(group):
+            if group in self.groups:
                 nbundles, ngroups, ncategories = self.groups[group]
             else:
                 nbundles, ngroups, ncategories = ([], [group], {})
-            [newbundles.append(b) for b in nbundles if b not in newbundles]
-            [newgroups.append(g) for g in ngroups if g not in newgroups]
+            [newbundles.add(b) for b in nbundles if b not in newbundles]
+            [newgroups.add(g) for g in ngroups if g not in newgroups]
             newcategories.update(ncategories)
-        groupscopy = copy.deepcopy(self.groups)
-        clientscopy = copy.deepcopy(self.clients)
-        return ClientMetadata(client, newgroups, newbundles, newcategories,
-                              probed, uuid, password, (groupscopy, clientscopy))
-        
-    def GetProbes(self, meta, force=False):
-        '''Return a set of probes for execution on client'''
-        if self.probes:
-            return self.probes.get_probe_data(meta)
-        return []
+        return ClientMetadata(client, profile, newgroups, newbundles, aliases,
+                              addresses, newcategories, uuid, password, self.query)
 
-    def ReceiveData(self, client, datalist):
-        self.cgroups[client.hostname] = []
-        self.probedata[client.hostname] = {}
-        for data in datalist:
-            self.ReceiveDataItem(client, data)
-        self.pdirty = True
-        self.write_probedata()
+    def get_all_group_names(self):
+        all_groups = set()
+        [all_groups.update(g[1]) for g in self.groups.values()]
+        return all_groups
 
-    def ReceiveDataItem(self, client, data):
-        '''Receive probe results pertaining to client'''
-        if not self.cgroups.has_key(client.hostname):
-            self.cgroups[client.hostname] = []
-        if data.text == None:
-            self.logger.error("Got null response to probe %s from %s" % \
-                              (data.get('name'), client.hostname))
-            try:
-                self.probedata[client.hostname].update({data.get('name'): ''})
-            except KeyError:
-                self.probedata[client.hostname] = {data.get('name'): ''}
-            return
-        dlines = data.text.split('\n')
-        self.logger.debug("%s:probe:%s:%s" % (client.hostname, 
-            data.get('name'), [line.strip() for line in dlines]))
-        for line in dlines[:]:
-            if line.split(':')[0] == 'group':
-                newgroup = line.split(':')[1].strip()
-                if newgroup not in self.cgroups[client.hostname]:
-                    self.cgroups[client.hostname].append(newgroup)
-                dlines.remove(line)
-        dtext = "\n".join(dlines)
-        try:
-            self.probedata[client.hostname].update({ data.get('name'):dtext })
-        except KeyError:
-            self.probedata[client.hostname] = { data.get('name'):dtext }
+    def get_client_names_by_profiles(self, profiles):
+        return [client for client, profile in self.clients.iteritems() \
+                if profile in profiles]
 
-    def AuthenticateConnection(self, user, password, address):
-        '''This function checks user and password'''
-        if user == 'root':
-            # we aren't using per-client keys
+    def get_client_names_by_groups(self, groups):
+        gprofiles = [profile for profile in self.profiles if \
+                     self.groups[profile][1].issuperset(groups)]
+        return self.get_client_names_by_profiles(gprofiles)
+
+    def merge_additional_groups(self, imd, groups):
+        for group in groups:
+            if group in self.categories and \
+                   self.categories[group] in imd.categories:
+                continue
+            nb, ng, _ = self.groups.get(group, (list(), [group], dict()))
+            for b in nb:
+                if b not in imd.bundles:
+                    imd.bundles.add(b)
+            for g in ng:
+                if g not in imd.groups:
+                    if g in self.categories and \
+                       self.categories[g] in imd.categories:
+                        continue
+                    if g in self.private:
+                        self.logger.error("Refusing to add dynamic membership in private group %s for client %s" % (g, imd.hostname))
+                        continue
+                    imd.groups.add(g)
+
+    def merge_additional_data(self, imd, source, data):
+        if not hasattr(imd, source):
+            setattr(imd, source, data)
+            imd.connectors.append(source)
+
+    def validate_client_address(self, client, addresspair):
+        '''Check address against client'''
+        address = addresspair[0]
+        if client in self.floating:
+            self.debug_log("Client %s is floating" % client)
+            return True
+        if address in self.addresses:
+            if client in self.addresses[address]:
+                self.debug_log("Client %s matches address %s" % (client, address))
+                return True
+            else:
+                self.logger.error("Got request for non-float client %s from %s" \
+                                  % (client, address))
+                return False
+        resolved = self.resolve_client(addresspair)
+        if resolved.lower() == client.lower():
+            return True
+        else:
+            self.logger.error("Got request for %s from incorrect address %s" \
+                              % (client, address))
+            self.logger.error("Resolved to %s" % resolved)
+            return False
+
+    def AuthenticateConnection(self, cert, user, password, address):
+        '''This function checks auth creds'''
+        if cert:
+            id_method = 'cert'
+            certinfo = dict([x[0] for x in cert['subject']])
+            # look at cert.cN
+            client = certinfo['commonName']
+            self.debug_log("Got cN %s; using as client name" % client)
+            auth_type = self.auth.get(client, 'cert+password')
+        elif user == 'root':
+            id_method = 'address'
             try:
                 client = self.resolve_client(address)
             except MetadataConsistencyError:
-                self.logger.error("Client %s failed to authenticate due to metadata problem" % (address[0]))
+                self.logger.error("Client %s failed to resolve; metadata problem" % (address[0]))
                 return False
         else:
+            id_method = 'uuid'
             # user maps to client
             if user not in self.uuid:
                 client = user
@@ -542,17 +667,22 @@ class Metadata(Bcfg2.Server.Plugin.MetadataPlugin,
             else:
                 client = self.uuid[user]
 
-        # we have the client
-        if client not in self.floating and user != 'root':
-            if address[0] in self.addresses:
-                # we are using manual resolution
-                if client not in self.addresses[address[0]]:
-                    self.logger.error("Got request for non-floating UUID %s from %s" % (user, address[0]))
-                    return False
-            elif client != self.resolve_client(address):
-                self.logger.error("Got request for non-floating UUID %s from %s" \
-                                  % (user, address[0]))
-                return False
+        # we have the client name
+        self.debug_log("Authenticating client %s" % client)
+
+        # next we validate the address
+        if id_method == 'uuid':
+            addr_is_valid = True
+        else:
+            addr_is_valid = self.validate_client_address(client, address)
+
+        if not addr_is_valid:
+            return False
+
+        if id_method == 'cert' and auth_type != 'cert+password':
+            # we are done if cert+password not required
+            return True
+
         if client not in self.passwords:
             if client in self.secure:
                 self.logger.error("Client %s in secure mode but has no password" % (address[0]))
@@ -561,7 +691,7 @@ class Metadata(Bcfg2.Server.Plugin.MetadataPlugin,
                 self.logger.error("Client %s used incorrect global password" % (address[0]))
                 return False
         if client not in self.secure:
-            if self.passwords.has_key(client):
+            if client in self.passwords:
                 plist = [self.password, self.passwords[client]]
             else:
                 plist = [self.password]
@@ -577,19 +707,19 @@ class Metadata(Bcfg2.Server.Plugin.MetadataPlugin,
                 return False
         # populate the session cache
         if user != 'root':
-            self.session_cache[address] = (time.time(), user)
+            self.session_cache[address] = (time.time(), client)
         return True
 
-    def GetClientByGroup(self, group):
-        '''Return a list of clients that are in a given group'''
-        return [client for client in self.clients \
-                if group in self.groups[self.clients[client]][1]]
+    def process_statistics(self, meta, _):
+        '''Hook into statistics interface to toggle clients in bootstrap mode'''
+        client = meta.hostname
+        if client in self.auth and self.auth[client] == 'bootstrap':
+            self.logger.info("Asserting client %s auth mode to cert" % client)
+            cli = self.clientdata_original.xpath('.//Client[@name="%s"]' \
+                                                 % (client))
+            cli[0].set('auth', 'cert')
+            self.write_back_clients()
 
-    def GetClientByProfile(self, profile):
-        '''Return a list of clients that are members of a given profile'''
-        return [client for client in self.clients \
-                if self.clients[client] == profile]
-    
     def viz(self, hosts, bundles, key, colors):
         '''admin mode viz support'''
         groups_tree = lxml.etree.parse(self.data + "/groups.xml")
@@ -599,7 +729,7 @@ class Metadata(Bcfg2.Server.Plugin.MetadataPlugin,
         viz_str = ""
         egroups = groups.findall("Group") + groups.findall('.//Groups/Group')
         for group in egroups:
-            if not categories.has_key(group.get('category')):
+            if not group.get('category') in categories:
                 categories[group.get('category')] = colors.pop()
             group.set('color', categories[group.get('category')])
         if None in categories:
@@ -607,7 +737,7 @@ class Metadata(Bcfg2.Server.Plugin.MetadataPlugin,
         if hosts:
             clients = self.clients
             for client, profile in clients.iteritems():
-                if instances.has_key(profile):
+                if profile in instances:
                     instances[profile].append(client)
                 else:
                     instances[profile] = [client]
@@ -624,7 +754,7 @@ class Metadata(Bcfg2.Server.Plugin.MetadataPlugin,
                  if bund.get('name') not in bundles]
             bundles.sort()
             for bundle in bundles:
-                viz_str +=  '''\t"bundle-%s" [ label="%s", shape="septagon"];\n''' \
+                viz_str += '''\t"bundle-%s" [ label="%s", shape="septagon"];\n''' \
                     % (bundle, bundle)
         gseen = []
         for group in egroups:
