@@ -1,18 +1,12 @@
 '''This module implements a package management scheme for all images'''
-__revision__ = '$Revision: 4953 $'
+__revision__ = '$Revision: 5477 $'
 
-import logging, re, Bcfg2.Server.Plugin
+import logging
+import re
+import Bcfg2.Server.Plugin
 
 logger = logging.getLogger('Bcfg2.Plugins.Pkgmgr')
 
-try:
-    pdlist = set
-except:
-    class pdlist(list):
-        def add(self, item):
-            if item not in self:
-                self.append(item)
-
 class FuzzyDict(dict):
     fuzzy = re.compile('(?P<name>.*):(?P<alist>\S+(,\S+)*)')
     def __getitem__(self, key):
@@ -61,7 +55,7 @@ class PNode(Bcfg2.Server.Plugin.INode):
     def __init__(self, data, pdict, parent=None):
         # copy local attributes to all child nodes if no local attribute exists
         if not pdict.has_key('Package'):
-            pdict['Package'] = pdlist()
+            pdict['Package'] = set()
         for child in data.getchildren():
             for attr in [key for key in data.attrib.keys() \
                          if key != 'name' and not child.attrib.has_key(key)]:
@@ -124,8 +118,8 @@ class PkgSrc(Bcfg2.Server.Plugin.XMLSrc):
 
 class Pkgmgr(Bcfg2.Server.Plugin.PrioDir):
     '''This is a generator that handles package assignments'''
-    __name__ = 'Pkgmgr'
-    __version__ = '$Id: Pkgmgr.py 4953 2008-10-28 03:02:33Z desai $'
+    name = 'Pkgmgr'
+    __version__ = '$Id: Pkgmgr.py 5477 2009-10-07 14:46:08Z solj $'
     __author__ = 'bcfg-dev@mcs.anl.gov'
     __child__ = PkgSrc
     __element__ = 'Package'
@@ -153,4 +147,9 @@ class Pkgmgr(Bcfg2.Server.Plugin.PrioDir):
                 [entry.remove(inst) for inst in \
                  entry.findall('Instance') \
                  if inst.get('arch') not in arches]
-                
+
+    def HandlesEntry(self, entry, metadata):
+        return entry.tag == 'Package' and entry.get('name').split(':')[0] in self.Entries['Package'].keys()
+
+    def HandleEntry(self, entry, metadata):
+        self.BindEntry(entry, metadata)
