@@ -1,10 +1,11 @@
-'''This generator provides service mappings'''
-__revision__ = '$Revision: 4490 $'
+'''This generator provides rule-based entry mappings'''
+__revision__ = '$Revision: 5477 $'
 
 import Bcfg2.Server.Plugin
 
+
 class Rules(Bcfg2.Server.Plugin.PrioDir):
     '''This is a generator that handles service assignments'''
-    __name__ = 'Rules'
-    __version__ = '$Id: Rules.py 4490 2008-04-08 17:22:39Z desai $'
+    name = 'Rules'
+    __version__ = '$Id: Rules.py 5477 2009-10-07 14:46:08Z solj $'
     __author__ = 'bcfg-dev@mcs.anl.gov'
