@@ -1,30 +1,33 @@
 '''This module implements a templating generator based on Genshi'''
-__revision__ = '$Revision: 4746 $'
+__revision__ = '$Revision: 5477 $'
 
-import Bcfg2.Server.Plugin, Bcfg2.Server.Plugins.TGenshi
-import lxml.etree, logging
 import genshi.template
+import lxml.etree
+import logging
+
+import Bcfg2.Server.Plugin
+import Bcfg2.Server.Plugins.TGenshi
 
 logger = logging.getLogger('Bcfg2.Plugins.SGenshi')
 
+
 class SGenshiTemplateFile(Bcfg2.Server.Plugins.TGenshi.TemplateFile):
+
     def get_xml_value(self, metadata):
-        stream = self.template.generate(metadata=metadata,
-                                        properties=self.properties.properties).filter( \
+        if not hasattr(self, 'template'):
+            logger.error("No parsed template information for %s" % (self.name))
+            raise Bcfg2.Server.Plugin.PluginExecutionError
+        stream = self.template.generate(metadata=metadata,).filter( \
             Bcfg2.Server.Plugins.TGenshi.removecomment)
-        data = stream.render('xml')
+        data = stream.render('xml', strip_whitespace=False)
         return lxml.etree.XML(data)
 
+
 class SGenshiEntrySet(Bcfg2.Server.Plugin.EntrySet):
+
     def __init__(self, path, fam, encoding):
         fpattern = '\S+\.xml'
-        try:
-            properties = Bcfg2.Server.Plugin.TemplateProperties( \
-                    '%s/../etc/properties.xml' % (path), fam)
-        except:
-            properties = Bcfg2.Server.Plugin.FakeProperties()
-
-        Bcfg2.Server.Plugin.EntrySet.__init__(self, fpattern, path, properties,
+        Bcfg2.Server.Plugin.EntrySet.__init__(self, fpattern, path,
                                               SGenshiTemplateFile, encoding)
         fam.AddMonitor(path, self)
 
@@ -44,18 +47,22 @@ class SGenshiEntrySet(Bcfg2.Server.Plugin.EntrySet):
                 logger.error("SGenshi: Failed to template file %s" % entry.name)
         return ret
 
-class SGenshi(SGenshiEntrySet, Bcfg2.Server.Plugin.StructurePlugin):
+
+class SGenshi(SGenshiEntrySet,
+              Bcfg2.Server.Plugin.Plugin,
+              Bcfg2.Server.Plugin.Structure):
     '''The SGenshi plugin provides templated structures'''
-    __name__ = 'SGenshi'
-    __version__ = '$Id: SGenshi.py 4746 2008-07-02 15:53:04Z desai $'
+    name = 'SGenshi'
+    __version__ = '$Id: SGenshi.py 5477 2009-10-07 14:46:08Z solj $'
     __author__ = 'bcfg-dev@mcs.anl.gov'
+    deprecated = True
 
     def __init__(self, core, datastore):
         Bcfg2.Server.Plugin.Plugin.__init__(self, core, datastore)
+        Bcfg2.Server.Plugin.Structure.__init__(self)
         try:
             SGenshiEntrySet.__init__(self, self.data, self.core.fam, core.encoding)
         except:
-            logger.error("Failed to load SGenshi repository; disabling SGenshi")
+            logger.error("Failed to load %s repository; disabling %s" \
+                         % (self.name, self.name))
             raise Bcfg2.Server.Plugin.PluginInitError
-        
-
