@@ -1,10 +1,18 @@
 '''This module manages ssh key files for bcfg2'''
-__revision__ = '$Revision: 4880 $'
+__revision__ = '$Revision: 5477 $'
 
-import binascii, os, socket, tempfile
+import binascii
+import os
+import socket
+import tempfile
+from subprocess import Popen, PIPE
 import Bcfg2.Server.Plugin
 
-class SSHbase(Bcfg2.Server.Plugin.GeneratorPlugin,  Bcfg2.Server.Plugin.DirectoryBacked):
+
+class SSHbase(Bcfg2.Server.Plugin.Plugin,
+              Bcfg2.Server.Plugin.Generator,
+              Bcfg2.Server.Plugin.DirectoryBacked,
+              Bcfg2.Server.Plugin.PullTarget):
     '''The sshbase generator manages ssh host keys (both v1 and v2)
     for hosts.  It also manages the ssh_known_hosts file. It can
     integrate host keys from other management domains and similarly
@@ -21,9 +29,9 @@ class SSHbase(Bcfg2.Server.Plugin.GeneratorPlugin,  Bcfg2.Server.Plugin.Director
       public key for (hostname)
     ssh_known_hosts -> the current known hosts file. this
       is regenerated each time a new key is generated.
-'''
-    __name__ = 'SSHbase'
-    __version__ = '$Id: SSHbase.py 4880 2008-08-19 03:11:18Z desai $'
+    '''
+    name = 'SSHbase'
+    __version__ = '$Id: SSHbase.py 5477 2009-10-07 14:46:08Z solj $'
     __author__ = 'bcfg-dev@mcs.anl.gov'
 
     pubkeys = ["ssh_host_dsa_key.pub.H_%s",
@@ -31,49 +39,69 @@ class SSHbase(Bcfg2.Server.Plugin.GeneratorPlugin,  Bcfg2.Server.Plugin.Director
     hostkeys = ["ssh_host_dsa_key.H_%s",
                 "ssh_host_rsa_key.H_%s", "ssh_host_key.H_%s"]
     keypatterns = ['ssh_host_dsa_key', 'ssh_host_rsa_key', 'ssh_host_key',
-                   'ssh_host_dsa_key.pub', 'ssh_host_rsa_key.pub', 'ssh_host_key.pub']
+                   'ssh_host_dsa_key.pub', 'ssh_host_rsa_key.pub',
+                   'ssh_host_key.pub']
 
     def __init__(self, core, datastore):
         Bcfg2.Server.Plugin.Plugin.__init__(self, core, datastore)
+        Bcfg2.Server.Plugin.Generator.__init__(self)
+        Bcfg2.Server.Plugin.PullTarget.__init__(self)
         try:
-            Bcfg2.Server.Plugin.DirectoryBacked.__init__(self, self.data, self.core.fam)
+            Bcfg2.Server.Plugin.DirectoryBacked.__init__(self, self.data,
+                                                         self.core.fam)
         except OSError, ioerr:
-            self.logger.error("Failed to load SSHbase repository from %s" % (self.data))
+            self.logger.error("Failed to load SSHbase repository from %s" \
+                              % (self.data))
             self.logger.error(ioerr)
             raise Bcfg2.Server.Plugin.PluginInitError
         self.Entries = {'ConfigFile':
-                        {'/etc/ssh/ssh_known_hosts':self.build_skn, 
-                         '/etc/ssh/ssh_host_dsa_key':self.build_hk,
-                         '/etc/ssh/ssh_host_rsa_key':self.build_hk,
-                         '/etc/ssh/ssh_host_dsa_key.pub':self.build_hk,
-                         '/etc/ssh/ssh_host_rsa_key.pub':self.build_hk,
-                         '/etc/ssh/ssh_host_key':self.build_hk,
-                         '/etc/ssh/ssh_host_key.pub':self.build_hk}}
+                        {'/etc/ssh/ssh_known_hosts': self.build_skn,
+                         '/etc/ssh/ssh_host_dsa_key': self.build_hk,
+                         '/etc/ssh/ssh_host_rsa_key': self.build_hk,
+                         '/etc/ssh/ssh_host_dsa_key.pub': self.build_hk,
+                         '/etc/ssh/ssh_host_rsa_key.pub': self.build_hk,
+                         '/etc/ssh/ssh_host_key': self.build_hk,
+                         '/etc/ssh/ssh_host_key.pub': self.build_hk}}
         self.ipcache = {}
         self.__skn = False
 
     def get_skn(self):
         '''build memory cache of the ssh known hosts file'''
         if not self.__skn:
-            static_entries = [key for key in self.entries \
-                              if key.endswith('.static')]
-            if static_entries:
-                self.__skn = "\n".join([self.entries[key].data for \
-                                        key in static_entries])
-            else:
-                self.__skn = ''
+            self.__skn = "\n".join([value.data for key, value in \
+                                    self.entries.iteritems() if \
+                                    key.endswith('.static')])
+            names = dict()
+            # if no metadata is registered yet, defer
+            if len(self.core.metadata.query.all()) == 0:
+                self.__skn = False
+                return self.__skn
+            for cmeta in self.core.metadata.query.all():
+                names[cmeta.hostname] = set([cmeta.hostname])
+                names[cmeta.hostname].update(cmeta.aliases)
+                newnames = set()
+                for name in names[cmeta.hostname]:
+                    newnames.add(name.split('.')[0])
+                    try:
+                        newnames.add(self.get_ipcache_entry(name)[0])
+                    except:
+                        continue
+                names[cmeta.hostname].update(newnames)
+                names[cmeta.hostname].update(cmeta.addresses)
+            # now we have our name cache
             pubkeys = [pubk for pubk in self.entries.keys() \
                        if pubk.find('.pub.H_') != -1]
             pubkeys.sort()
+            badnames = set()
             for pubkey in pubkeys:
                 hostname = pubkey.split('H_')[1]
-                try:
-                    (ipaddr, fqdn) = self.get_ipcache_entry(hostname)
-                except socket.gaierror:
+                if hostname not in names:
+                    if hostname not in badnames:
+                        badnames.add(hostname)
+                        self.logger.error("SSHbase: Unknown host %s; ignoring public keys" % hostname)
                     continue
-                shortname = hostname.split('.')[0]
-                self.__skn += "%s,%s,%s %s" % (shortname, fqdn, ipaddr,
-                                               self.entries[pubkey].data)
+                self.__skn += "%s %s" % (','.join(names[hostname]),
+                                         self.entries[pubkey].data)
         return self.__skn
 
     def set_skn(self, value):
@@ -88,11 +116,11 @@ class SSHbase(Bcfg2.Server.Plugin.GeneratorPlugin,  Bcfg2.Server.Plugin.Director
             self.skn = False
         if event and event.filename.endswith('.static'):
             self.skn = False
-        if not self.skn:
-            if (len(self.entries.keys())) > (0.95 * len(os.listdir(self.data))):
-                a = self.skn
+        if not self.__skn:
+            if (len(self.entries.keys())) >= (len(os.listdir(self.data))-1):
+                _ = self.skn
 
-    def HandlesEntry(self, entry):
+    def HandlesEntry(self, entry, _):
         '''Handle key entries dynamically'''
         return entry.tag == 'ConfigFile' and \
                ([fpat for fpat in self.keypatterns
@@ -108,7 +136,7 @@ class SSHbase(Bcfg2.Server.Plugin.GeneratorPlugin,  Bcfg2.Server.Plugin.Director
 
     def get_ipcache_entry(self, client):
         '''build a cache of dns results'''
-        if self.ipcache.has_key(client):
+        if client in self.ipcache:
             if self.ipcache[client]:
                 return self.ipcache[client]
             else:
@@ -120,7 +148,9 @@ class SSHbase(Bcfg2.Server.Plugin.GeneratorPlugin,  Bcfg2.Server.Plugin.Director
                 self.ipcache[client] = (ipaddr, client)
                 return (ipaddr, client)
             except socket.gaierror:
-                ipaddr = os.popen("getent hosts %s" % client).read().strip().split()
+                cmd = "getent hosts %s" % client
+                ipaddr = Popen(cmd, shell=True, \
+                               stdout=PIPE).stdout.read().strip().split()
                 if ipaddr:
                     self.ipcache[client] = (ipaddr, client)
                     return (ipaddr, client)
@@ -133,7 +163,7 @@ class SSHbase(Bcfg2.Server.Plugin.GeneratorPlugin,  Bcfg2.Server.Plugin.Director
         client = metadata.hostname
         entry.text = self.skn
         hostkeys = [keytmpl % client for keytmpl in self.pubkeys \
-                        if self.entries.has_key(keytmpl % client)]
+                        if (keytmpl % client) in self.entries]
         hostkeys.sort()
         for hostkey in hostkeys:
             entry.text += "localhost,localhost.localdomain,127.0.0.1 %s" % (
@@ -147,7 +177,7 @@ class SSHbase(Bcfg2.Server.Plugin.GeneratorPlugin,  Bcfg2.Server.Plugin.Director
         filename = "%s.H_%s" % (entry.get('name').split('/')[-1], client)
         if filename not in self.entries.keys():
             self.GenerateHostKeys(client)
-        if not self.entries.has_key(filename):
+        if not filename in self.entries:
             self.logger.error("%s still not registered" % filename)
             raise Bcfg2.Server.Plugin.PluginExecutionError
         keydata = self.entries[filename].data
@@ -175,15 +205,18 @@ class SSHbase(Bcfg2.Server.Plugin.GeneratorPlugin,  Bcfg2.Server.Plugin.Director
 
             if hostkey not in self.entries.keys():
                 fileloc = "%s/%s" % (self.data, hostkey)
-                publoc = self.data + '/' + ".".join([hostkey.split('.')[0]]+['pub', "H_%s" % client])
+                publoc = self.data + '/' + ".".join([hostkey.split('.')[0],
+                                                     'pub',
+                                                     "H_%s" % client])
                 tempdir = tempfile.mkdtemp()
                 temploc = "%s/%s" % (tempdir, hostkey)
-                os.system('ssh-keygen -q -f %s -N "" -t %s -C root@%s < /dev/null' %
-                          (temploc, keytype, client))
+                cmd = 'ssh-keygen -q -f %s -N "" -t %s -C root@%s < /dev/null'
+                os.system(cmd % (temploc, keytype, client))
                 open(fileloc, 'w').write(open(temploc).read())
                 open(publoc, 'w').write(open("%s.pub" % temploc).read())
                 self.AddEntry(hostkey)
-                self.AddEntry(".".join([hostkey.split('.')[0]]+['pub', "H_%s" % client]))
+                self.AddEntry(".".join([hostkey.split('.')[0]]+['pub', "H_%s" \
+                                                                % client]))
                 try:
                     os.unlink(temploc)
                     os.unlink("%s.pub" % temploc)
