@@ -1,10 +1,12 @@
 '''This generator provides service mappings'''
-__revision__ = '$Revision: 1827 $'
+__revision__ = '$Revision: 5477 $'
 
 import Bcfg2.Server.Plugin
 
+
 class Svcmgr(Bcfg2.Server.Plugin.PrioDir):
     '''This is a generator that handles service assignments'''
-    __name__ = 'Svcmgr'
-    __version__ = '$Id: Svcmgr.py 1827 2006-04-10 20:01:43Z desai $'
+    name = 'Svcmgr'
+    __version__ = '$Id: Svcmgr.py 5477 2009-10-07 14:46:08Z solj $'
     __author__ = 'bcfg-dev@mcs.anl.gov'
+    deprecated = True
