@@ -22,14 +22,14 @@ def config_item_modified(request, eyedee =None, timestamp = 'now', type=TYPE_MOD
     else:
         mod_or_bad = "bad"
     
-    item = Entries_interactions.objects.get(id=eyedee).entry
+    item = get_object_or_404(Entries_interactions, id=eyedee)
     #if everything is blank except current_exists, do something special
     cursor = connection.cursor()
     if timestamp == 'now':
         cursor.execute("select client_id from reports_interaction, reports_entries_interactions, reports_client "+
                    "WHERE reports_client.current_interaction_id = reports_entries_interactions.interaction_id "+
                    "AND reports_entries_interactions.interaction_id = reports_interaction.id "+
-                   "AND reports_entries_interactions.entry_id = %s " +
+                   "AND reports_entries_interactions.id = %s " +
                    "AND reports_entries_interactions.type = %s", [eyedee, type])
         associated_client_list = Client.objects.active(timestamp).filter(id__in=[x[0] for x in cursor.fetchall()])
     else:
@@ -91,8 +91,8 @@ def modified_item_index(request, timestamp = 'now'):
 def client_index(request, timestamp = 'now'):
     timestamp = timestamp.replace("@"," ")
     client_list = Client.objects.active(timestamp).order_by('name')
-    client_list_b = client_list[:len(client_list)/2]
     client_list_a = client_list[len(client_list)/2:]
+    client_list_b = client_list[:len(client_list)/2]
     if timestamp == 'now':
         timestamp = datetime.now().isoformat('@')
     return render_to_response('clients/index.html',
@@ -256,9 +256,11 @@ def prepare_client_lists(request, timestamp = 'now'):
         cursor.execute("select client_id, MAX(timestamp) as timestamp from reports_interaction GROUP BY client_id")
         results = cursor.fetchall()
         for x in results:
-            if type(x[1]) == type(""):
-                x[1] = util.typecast_timestamp(x[1])
-        stale_all_client_list = Client.objects.active(timestamp).filter(id__in=[x[0] for x in results if datetime.now() - x[1] > timedelta(days=1)])
+            if type(x[1]) == type("") or type(x[1]) == type(u""):
+                ts = util.typecast_timestamp(x[1])
+            else:
+                ts = x[1]
+        stale_all_client_list = Client.objects.active(timestamp).filter(id__in=[x[0] for x in results if datetime.now() - ts > timedelta(days=1)])
     else:
         cursor.execute("select client_id, timestamp, MAX(timestamp) as timestamp from reports_interaction "+
                        "WHERE timestamp < %s GROUP BY client_id", [timestamp])
