@@ -19,5 +19,5 @@ cd /tmp
 
 tar czf "${tarname}" "${name}-${version}"
 gpg --armor --output "${tarname}".gpg --detach-sig "${tarname}"
-scp "${tarname}"* terra.mcs.anl.gov:/nfs/ftp/pub/bcfg
+scp "${tarname}"* terra.mcs.anl.gov:/mcs/ftp/pub/bcfg
 
